from ftw.book.interfaces import IWithinBookLayer
from ftw.book.testing import ZCML_LAYER
from ftw.testing import MockTestCase
from simplelayout.base.interfaces import ISimpleViewletProvider
from ftw.book.interfaces import IBookTextBlock
from zope.browser.interfaces import IBrowserView
from zope.component import getMultiAdapter
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.viewlet.interfaces import IViewlet


class TestTableValidationViewlet(MockTestCase):

    layer = ZCML_LAYER

    def create_stubs(self, text):
        context = self.providing_stub([IBookTextBlock])
        self.expect(context.getText()).result(text)

        request = self.stub_request([IWithinBookLayer,
                                     IUserPreferredLanguages])
        self.expect(request.getPreferredLanguages()).result('en')

        view = self.providing_stub(IBrowserView)
        manager = self.providing_stub([ISimpleViewletProvider])
        return context, request, view, manager

    def test_invisible_when_valid(self):
        stubs = self.create_stubs('Hello World')
        self.replay()

        viewlet = getMultiAdapter(stubs, IViewlet,
            name='ftw.book.textblock.validation')

        viewlet.update()
        self.assertEqual(viewlet.render().strip(), '')

    def test_validates_table_width(self):
        html = '\n'.join((
                '<table>',
                '<tr>',
                '<td>foo</td>',
                '</tr>',
                '</table>'))

        stubs = self.create_stubs(html)
        self.replay()

        viewlet = getMultiAdapter(stubs, IViewlet,
            name='ftw.book.textblock.validation')

        viewlet.update()
        viewlet_html = viewlet.render().strip()

        self.assertIn(
            'Please specify the width of the table columns / cells.',
            viewlet_html)

    def test_validates_multiple_table_widths(self):
        html = '\n'.join((
                '<table>',
                '<tr><td width="100%">foo</td></tr>',
                '</table>',
                'bar',
                '<table>',
                '<tr><td>bar</td></tr>',
                '</table>'))

        stubs = self.create_stubs(html)
        self.replay()

        viewlet = getMultiAdapter(stubs, IViewlet,
            name='ftw.book.textblock.validation')

        viewlet.update()
        viewlet_html = viewlet.render().strip()

        self.assertIn(
            'Please specify the width of the table columns / cells.',
            viewlet_html)



from ftw.book.interfaces import IWithinBookLayer
from ftw.builder import builder_registry
from ftw.builder.archetypes import ArchetypesBuilder
from ftw.pdfgenerator.utils import provide_request_layer


class BookBuilder(ArchetypesBuilder):

    portal_type = 'Book'

    def __init__(self, *args, **kwargs):
        super(BookBuilder, self).__init__(*args, **kwargs)
        self.apply_layer = None

    def with_layout(self, layout_iface):
        dotted_name = '.'.join((layout_iface.__module__,
                                layout_iface.__name__))
        self.having(latex_layout=dotted_name)
        self.apply_layer = layout_iface
        return self

    def after_create(self, obj):
        provide_request_layer(obj.REQUEST, IWithinBookLayer)
        if self.apply_layer:
            provide_request_layer(obj.REQUEST, self.apply_layer)
        super(BookBuilder, self).after_create(obj)


builder_registry.register('book', BookBuilder)


class ChapterBuilder(ArchetypesBuilder):

    portal_type = 'Chapter'

builder_registry.register('chapter', ChapterBuilder)


class TextBlockBuilder(ArchetypesBuilder):

    portal_type = 'BookTextBlock'

builder_registry.register('book textblock', TextBlockBuilder)


class HTMLBlockBuilder(ArchetypesBuilder):

    portal_type = 'HTMLBlock'

builder_registry.register('htmlblock', HTMLBlockBuilder)


class RemarkBuilder(ArchetypesBuilder):

    portal_type = 'Remark'

builder_registry.register('remark', RemarkBuilder)


class ListingBlockBuilder(ArchetypesBuilder):

    portal_type = 'ListingBlock'

builder_registry.register('listingblock', ListingBlockBuilder)


class TableBuilder(ArchetypesBuilder):

    portal_type = 'Table'

    def __init__(self, *args, **kwargs):
        super(TableBuilder, self).__init__(*args, **kwargs)
        self.table = None

    def with_table(self, table):
        """Fills the table with data represented as list of lists.
        """
        self.table = table
        return self

    def with_dummy_table(self):
        return self.with_table(zip(map(str, range(0, 10, 2)),
                                   map(str, range(1, 10, 2))))

    def after_create(self, obj):
        self._update_table_data(obj, self.table)
        super(TableBuilder, self).after_create(obj)

    def _update_table_data(self, obj, table):
        if not table:
            return

        column_properties = obj.getColumnProperties()
        for column_index in range(len(table[0])):
            column_properties[column_index]['active'] = True

        data = map(lambda row: dict([('column_%i' % num, value)
                                     for num, value in enumerate(row)]),
                   table)
        obj.setData(data)

builder_registry.register('table', TableBuilder)



from Products.ATContentTypes.browser import nextprevious
from ftw.book.interfaces import IBook
from plone.app.layout.nextprevious.interfaces import INextPreviousProvider
from zope.component import adapts
from zope.interface import implements


class BookNextPrevious(nextprevious.ATFolderNextPrevious):
    """Let a folder act as a next/previous provider. This will be
    automatically found by the @@plone_nextprevious_view and viewlet.
    """

    implements(INextPreviousProvider)
    adapts(IBook)

    def buildNextPreviousQuery(self, *args, **kwargs):
        query = super(BookNextPrevious, self).buildNextPreviousQuery(
            *args, **kwargs)
        del query['is_folderish']
        return query



"""Common configuration constants
"""


PROJECTNAME = 'ftw.book'


ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'Book': 'ftw.book: Add Book',
    'Chapter': 'ftw.book: Add Chapter',
    'HTMLBlock': 'ftw.book: Add HTML Block',
    'Remark': 'ftw.book: Add Remark',
    'Table': 'ftw.book: Add Table',
    'BookTextBlock': 'ftw.book: Add Book Text Block',
    }


# Book LaTeX layout registry. Do not change manually, use
# ftw.book.latex.utils.register_book_layout
BOOK_LAYOUT_REGISTRY = {}


INDEXES = (('book_keywords', 'KeywordIndex'),
           )



from collective.transmogrifier import transmogrifier
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.latex.defaultlayout import IDefaultBookLayoutSelectionLayer
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.pdfgenerator.utils import provide_request_layer
from ftw.testing.layer import ComponentRegistryLayer
from plone.app.testing import IntegrationTesting, FunctionalTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import ploneSite
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.browserlayer.layer import mark_layer
from plone.mocktestcase.dummy import Dummy
from plone.testing import Layer
from plone.testing import z2
from plone.testing import zca
from plone.testing import zodb
from zope.configuration import xmlconfig
import ftw.book.tests.builders
import ftw.contentpage.tests.builders


def clear_transmogrifier_registry():
    transmogrifier.configuration_registry._config_info = {}
    transmogrifier.configuration_registry._config_ids = []


class LatexZCMLLayer(Layer):
    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    defaultBases = (zca.ZCML_DIRECTIVES, )

    def testSetUp(self):
        import ftw.book.tests
        self['configurationContext'] = zca.stackConfigurationContext(
            self.get('configurationContext'))

        xmlconfig.file('latex.zcml', ftw.book.tests,
                       context=self['configurationContext'])

    def testTearDown(self):
        del self['configurationContext']


LATEX_ZCML_LAYER = LatexZCMLLayer()


class ZCMLLayer(ComponentRegistryLayer):
    """Test layer loading the complete package ZCML.
    """

    def setUp(self):
        super(ZCMLLayer, self).setUp()
        import ftw.book.tests
        self.load_zcml_file('test.zcml', ftw.book.tests)


ZCML_LAYER = ZCMLLayer()


class FtwBookLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.book')
        z2.installProduct(app, 'simplelayout.base')
        z2.installProduct(app, 'ftw.contentpage')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.book:default')
        applyProfile(portal, 'ftw.tabbedview:default')
        applyProfile(portal, 'ftw.zipexport:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

    def tearDown(self):
        super(FtwBookLayer, self).tearDown()
        clear_transmogrifier_registry()


FTW_BOOK_FIXTURE = FtwBookLayer()
FTW_BOOK_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_BOOK_FIXTURE, ), name="ftw.book:Integration")
FTW_BOOK_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_BOOK_FIXTURE,
           set_builder_session_factory(functional_session_factory)
           ), name="ftw.book:Functional")


class ExampleContentLayer(Layer):

    defaultBases = (FTW_BOOK_FIXTURE, )

    def setUp(self):
        # Stack the component registry
        self['configurationContext'] = zca.stackConfigurationContext(
            self.get('configurationContext'))

        # Stack the database
        self['zodbDB'] = zodb.stackDemoStorage(
            self.get('zodbDB'), name='ftw.book:examplecontent')

        # Register and apply the example content GS profile
        import ftw.book.tests
        xmlconfig.file('examplecontent.zcml', ftw.book.tests,
                       context=self['configurationContext'])

        with ploneSite() as portal:
            request = portal.REQUEST
            mark_layer(None, Dummy(request=request))
            provide_request_layer(request, IWithinBookLayer)
            provide_request_layer(request, IDefaultBookLayoutSelectionLayer)

            applyProfile(portal, 'ftw.book.tests:examplecontent')

    def tearDown(self):
        # Zap the stacked ZODB
        self['zodbDB'].close()
        del self['zodbDB']
        # Zap the stacked component registry
        del self['configurationContext']


EXAMPLE_CONTENT_FIXTURE = ExampleContentLayer()
EXAMPLE_CONTENT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(EXAMPLE_CONTENT_FIXTURE, ),
    name="ftw.book:examplecontent:integration")



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content import folder
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IChapter
from ftw.contentpage.content.schema import finalize
from simplelayout.base.interfaces import ISimpleLayoutBlock
from simplelayout.base.interfaces import ISimpleLayoutCapable
from zope.interface import implements


try:
    from Products.LinguaPlone.public import registerType
except ImportError:
    # No multilingual support
    from Products.Archetypes.atapi import registerType


chapter_schema = folder.ATFolder.schema.copy()
finalize(chapter_schema, hide=['description'])
chapter_schema['excludeFromNav'].default = False


class Chapter(folder.ATFolder):
    implements(IChapter, ISimpleLayoutCapable, ISimpleLayoutBlock)
    security = ClassSecurityInfo()

    schema = chapter_schema

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False


registerType(Chapter, PROJECTNAME)



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from unittest2 import TestCase



class TestHTMLBlock(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.book = create(Builder('book'))
        self.chapter = create(Builder('chapter').within(self.book))

    @browsing
    def test_creating_htmlblock(self, browser):
        browser.login().visit(self.chapter)
        factoriesmenu.add('HTML Block')

        browser.fill({
                'Title': 'The HTML Block',
                'Show Title': True,
                'HTML': '<p>Some <b>body</b> text</p>'}).submit()
        self.assertEquals(self.chapter.absolute_url() + '/#the-html-block',
                          browser.url)

        blocks = browser.css('.BlockOverallWrapper.html-block')
        self.assertEquals(1, len(blocks),
                          'Expected chapter to have exactly one block')

        block, = blocks
        self.assertEquals(['1.1 The HTML Block'], block.css('h3').text,
                          'Expected block title to be visible.')

        self.assertEquals(
            '<p>Some <b>body</b> text</p>',
            block.css('.sl-text-wrapper').first.normalized_innerHTML)

    @browsing
    def test_showing_block_title(self, browser):
        create(Builder('htmlblock')
               .titled('Visible')
               .having(showTitle=True)
               .within(self.chapter))

        create(Builder('htmlblock')
               .titled('Hidden')
               .having(showTitle=False)
               .within(self.chapter))

        browser.login().visit(self.chapter)
        visible, hidden = browser.css('.BlockOverallWrapper.html-block')

        self.assertEquals(
            ['1.1 Visible'], visible.css('h3').text,
            'Expected block title of the "Visible" block to be visible.')

        self.assertEquals(
            [], hidden.css('h3').text,
            'Expected block title of the "Hidden" block to be hidden.')

    @browsing
    def test_hiding_title_from_table_of_contents(self, browser):
        create(Builder('htmlblock')
               .titled('Block in TOC')
               .having(showTitle=True,
                       hideFromTOC=False)
               .within(self.chapter))

        create(Builder('htmlblock')
                     .titled('Block NOT in TOC')
                     .having(showTitle=True,
                             hideFromTOC=True)
                     .within(self.chapter))

        browser.login().visit(self.chapter)

        self.assertEquals(
            ['1.1 Block in TOC', 'Block NOT in TOC'],
            browser.css('.BlockOverallWrapper.html-block h3').text,
            'Only the first block title should have a number prefix.')

    @browsing
    def test_latex_fields_available(self, browser):
        browser.login().open(self.chapter)
        factoriesmenu.add('HTML Block')

        form = browser.find('Title').parent('form')
        labels = form.field_labels

        self.assertIn('LaTeX code above content', labels)
        self.assertIn('LaTeX code beneath content', labels)

    @browsing
    def test_text_field_is_textarea_for_html(self, browser):
        browser.login().visit(self.chapter)
        factoriesmenu.add('HTML Block')
        self.assertEquals('textarea', browser.find('HTML').tag)



from ftw.book.interfaces import IRemark, IAddRemarkLayer
from ftw.pdfgenerator.view import MakoLaTeXView
from zope.component import adapts
from zope.interface import Interface


class RemarkLaTeXView(MakoLaTeXView):
    adapts(IRemark, IAddRemarkLayer, Interface)

    def render(self):

        self.layout.use_package('color')
        latex = []

        title = self.convert(self.context.Title())
        if title:
            latex.append('{\\bf %s}\\' % title)

        text = self.context.getText().strip()
        if len(text) > 0:
            latex.append(self.convert(text))

        latex.append('')

        return self.embed_in_grey_box('\n'.join(latex))

    def embed_in_grey_box(self, latex):
        """ Embed the given latex code in a grey box
        """

        color = r'\definecolor{light-gray}{gray}{0.8}'
        box_begin = r'\fcolorbox{black}{light-gray}{\parbox[r]{\textwidth}{'
        box_end = '}}'

        return color + box_begin + latex + box_end



from Products.Five.browser import BrowserView
from ftw.book.helpers import BookHelper
from ftw.contentpage.browser.textblock_view import TextBlockView


class BookTextBlockView(TextBlockView):

    def get_dynamic_title(self):
        return BookHelper()(self.context)


class HTMLBlockView(BrowserView):

    def get_dynamic_title(self):
        return BookHelper()(self.context)


class BookChapterView(BrowserView):

    def get_dynamic_title(self):
        return BookHelper()(self.context, linked=True)



from ftw.book.interfaces import IBook
from ftw.book.latex.defaultlayout import DefaultBookLayout
from ftw.book.latex.defaultlayout import IDefaultBookLayoutSelectionLayer
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.testing import MockTestCase
from mocker import ANY
from zope.component import getMultiAdapter, queryMultiAdapter
from zope.interface.verify import verifyClass


class TestDefaultBookLayout(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def _mock_book(self, data=None):
        options = {
            'Title': 'My book',
            'getUse_titlepage': True,
            'getUse_toc': True,
            'getUse_lot': True,
            'getUse_loi': True,
            'getUse_index': False,
            'author_address': 'Bern\nSwitzerland',
            'release': '2.5',
            'author': '4teamwork',
            'titlepage_logo': None,
            'titlepage_width': 0,
            }

        if data:
            options.update(data)

        if options.get('titlepage_logo', None):
            options['titlepage_logo'] = self.create_dummy(
                data=options['titlepage_logo'])

        book = self.providing_stub([IBook])
        for key, value in options.items():
            self.expect(getattr(book, key)()).result(value)

        for key in ['author_address', 'author', 'release', 'titlepage_logo',
                    'titlepage_logo_width']:
            value = options.get(key)
            self.expect(book.Schema().getField(key).get(book)).result(value)

        return book

    def _mock_portal_languages_tool(self):
        language_tool = self.stub()
        self.mock_tool(language_tool, 'portal_languages')
        self.expect(language_tool.getPreferredLanguage()).result('en')

    def test_component_is_registered(self):
        context = object()
        request = self.providing_stub([IDefaultBookLayoutSelectionLayer])
        builder = self.providing_stub([IBuilder])

        self.replay()
        component = getMultiAdapter((context, request, builder),
                                    ILaTeXLayout)

        self.assertNotEquals(component, None)
        self.assertEqual(type(component), DefaultBookLayout)

    def test_layout_only_registered_within_book(self):
        context = object()
        request = self.providing_stub([IDefaultBookLayoutSelectionLayer])
        builder = object()

        self.replay()
        component = queryMultiAdapter((context, request, builder),
                                    ILaTeXLayout)

        self.assertEqual(component, None)

    def test_layout_implements_interface(self):
        self.assertTrue(ILaTeXLayout.implementedBy(DefaultBookLayout))
        verifyClass(ILaTeXLayout, DefaultBookLayout)

    def test_get_book_walks_up(self):
        book = self.providing_stub([IBook])
        subchapter = self.set_parent(
            self.stub(), self.set_parent(
                self.stub(),
                book))

        self.replay()

        layout = DefaultBookLayout(subchapter, object(), object())

        self.assertEqual(layout.get_book(), book)

    def test_get_book_returns_None_if_not_within_book(self):
        root = self.stub()
        self.expect(root.__parent__, None)
        obj = self.set_parent(
            self.stub(),
            self.set_parent(
                self.stub(),
                root))

        self.replay()
        layout = DefaultBookLayout(obj, object(), object())

        self.assertEqual(layout.get_book(), None)

    def test_get_render_arguments(self):
        self._mock_portal_languages_tool()
        book = self._mock_book()
        self.replay()

        layout = DefaultBookLayout(book, object(), object())

        self.assertEqual(
            layout.get_render_arguments(),
            {'context_is_book': True,
             'title': 'My book',
             'use_titlepage': True,
             'use_toc': True,
             'use_lot': True,
             'use_loi': True,
             'use_index': False,
             'index_title': u'Index',
             'authoraddress': r'Bern\\Switzerland',
             'author': '4teamwork',
             'release': '2.5',
             'babel': 'english',
             'logo': False,
             'logo_width': 0})

    def test_get_render_arguments_babel(self):
        book = self._mock_book()

        language_tool = self.stub()
        self.mock_tool(language_tool, 'portal_languages')
        self.expect(language_tool.getPreferredLanguage()).result('de')

        self.replay()

        layout = DefaultBookLayout(book, object(), object())

        self.assertEqual(layout.get_render_arguments()['babel'], 'ngerman')

    def test_rendering_works(self):
        self._mock_portal_languages_tool()
        book = self._mock_book()
        builder = self.mocker.mock()

        self.expect(builder.add_file('sphinx.sty', data=ANY))
        self.expect(builder.add_file('fncychap.sty', data=ANY))
        self.expect(builder.add_file('sphinxftw.cls', data=ANY))
        self.expect(builder.add_file('sphinxhowto.cls', data=ANY))
        self.expect(builder.add_file('sphinxmanual.cls', data=ANY))
        self.replay()

        layout = DefaultBookLayout(book, object(), builder)

        latex = layout.render_latex('content latex')

        self.assertIn(r'My book', latex)
        self.assertIn(r'content latex', latex)
        self.assertIn(r'\maketitle', latex)
        self.assertIn(r'\tableofcontents', latex)
        self.assertIn(r'\listoffigures', latex)
        self.assertIn(r'\listoftables', latex)
        self.assertIn(r'\release{2.5}', latex)
        self.assertIn(r'\author{4teamwork}', latex)
        self.assertIn(r'\authoraddress{Bern\\Switzerland}', latex)

    def test_disabled_metadata(self):
        self._mock_portal_languages_tool()
        book = self._mock_book({
                'release': '',
                'author': '',
                'author_address': ''})
        builder = self.mocker.mock()

        self.expect(builder.add_file('sphinx.sty', data=ANY))
        self.expect(builder.add_file('fncychap.sty', data=ANY))
        self.expect(builder.add_file('sphinxftw.cls', data=ANY))
        self.expect(builder.add_file('sphinxhowto.cls', data=ANY))
        self.expect(builder.add_file('sphinxmanual.cls', data=ANY))
        self.replay()

        layout = DefaultBookLayout(book, object(), builder)

        latex = layout.render_latex('content latex')

        self.assertNotIn(r'\release{', latex)
        self.assertNotIn(r'\author', latex)
        self.assertNotIn(r'\authoraddress', latex)

    def test_logo_with_width(self):
        self._mock_portal_languages_tool()
        book = self._mock_book({
                'titlepage_logo': 'my-image',
                'titlepage_logo_width': 55})
        builder = self.mocker.mock()

        self.expect(builder.add_file('titlepage_logo.jpg', data='my-image'))

        self.expect(builder.add_file('sphinx.sty', data=ANY))
        self.expect(builder.add_file('fncychap.sty', data=ANY))
        self.expect(builder.add_file('sphinxftw.cls', data=ANY))
        self.expect(builder.add_file('sphinxhowto.cls', data=ANY))
        self.expect(builder.add_file('sphinxmanual.cls', data=ANY))
        self.replay()

        layout = DefaultBookLayout(book, object(), builder)

        latex = layout.render_latex('content latex')

        self.assertIn(
            r'\def\sphinxlogo{\includegraphics[width=.55\textwidth]{' + \
                r'titlepage_logo.jpg}}',
            latex)

    def test_logo_without_width(self):
        self._mock_portal_languages_tool()
        book = self._mock_book({
                'titlepage_logo': 'my-image',
                'titlepage_logo_width': 0})
        builder = self.mocker.mock()

        self.expect(builder.add_file('titlepage_logo.jpg', data='my-image'))

        self.expect(builder.add_file('sphinx.sty', data=ANY))
        self.expect(builder.add_file('fncychap.sty', data=ANY))
        self.expect(builder.add_file('sphinxftw.cls', data=ANY))
        self.expect(builder.add_file('sphinxhowto.cls', data=ANY))
        self.expect(builder.add_file('sphinxmanual.cls', data=ANY))
        self.replay()

        layout = DefaultBookLayout(book, object(), builder)

        latex = layout.render_latex('content latex')

        self.assertIn(
            r'\def\sphinxlogo{\includegraphics{' + \
                r'titlepage_logo.jpg}}',
            latex)

    def test_title_is_inserted_literally(self):
        # The spinx layout expects the title to not be LaTeX, but the string
        # literal.

        self._mock_portal_languages_tool()
        book = self._mock_book({'Title': 'Foo-Bar'})
        self.replay()

        layout = DefaultBookLayout(book, object(), object())

        self.assertEqual('Foo-Bar', layout.get_render_arguments()['title'])

    def test_using_index(self):
        self._mock_portal_languages_tool()
        book_without_index = self._mock_book({'getUse_index': False})
        book_with_index = self._mock_book({'getUse_index': True})
        builder = self.stub()
        self.expect(builder.add_file(ANY, data=ANY))
        self.replay()

        without_index_latex = DefaultBookLayout(
            book_without_index, object(), builder).render_latex('')
        self.assertNotIn(r'\makeindex', without_index_latex)
        self.assertNotIn(r'\printindex', without_index_latex)

        with_index_latex = DefaultBookLayout(
            book_with_index, object(), builder).render_latex('')
        self.assertIn(r'\makeindex', with_index_latex)
        self.assertIn(r'\printindex', with_index_latex)



from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import ILaTeXInjectionController
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.interfaces import NO_PREFERRED_LAYOUT
from ftw.book.interfaces import ONECOLUMN_LAYOUT
from ftw.book.interfaces import TWOCOLUMN_LAYOUT
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.pdfgenerator.layout.baselayout import BaseLayout
from ftw.testing import MockTestCase
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import Interface
from zope.interface import directlyProvides
from zope.interface.verify import verifyClass


class IFoo(Interface):
    pass


class TestInjectionAwareConvertObject(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestInjectionAwareConvertObject, self).setUp()

        context = self.create_dummy()
        request = self.providing_stub(IWithinBookLayer)
        builder = self.providing_stub(IBuilder)

        self.layout = BaseLayout(context, request, builder)

    def mock_extender_values(self, mock, **data):
        default_data = {'preLatexCode': '',
                        'postLatexCode': '',
                        'preferredColumnLayout': NO_PREFERRED_LAYOUT,
                        'latexLandscape': False,
                        'preLatexClearpage': False,
                        'postLatexClearpage': False,
                        'preLatexNewpage': False}
        default_data.update(data)
        data = default_data

        schema = self.stub()
        self.expect(mock.Schema()).result(schema).count(0, None)

        for fieldname, value in data.items():
            self.expect(schema.getField(fieldname).get(mock)).result(value)

        self.expect(mock.getPhysicalPath()).result(['', 'path', 'to', 'obj'])
        return schema

    def test_not_injected_without_interface(self):
        obj = self.mocker.mock()
        self.expect(obj.Schema()).count(0)

        self.replay()

        self.assertEqual(self.layout.render_latex_for(obj), '')

    def test_injected_with_interface(self):
        latex_pre_code = 'INJECTED PRE LATEX CODE'
        latex_post_code = 'INJECTED POST LATEX CODE'

        obj = self.providing_stub(ILaTeXCodeInjectionEnabled)

        self.mock_extender_values(obj, preLatexCode=latex_pre_code,
                                  postLatexCode=latex_post_code)

        self.replay()
        latex = self.layout.render_latex_for(obj)

        self.assertIn(latex_pre_code, latex)
        self.assertIn(latex_post_code, latex)
        self.assertIn('/'.join(obj.getPhysicalPath()), latex)

    def test_bad_schemaextender_state(self):
        # sometimes the field can not be retrieved. We do nothing and we
        # don't fail in this case.
        obj_dummy = self.create_dummy()
        directlyProvides(obj_dummy, ILaTeXCodeInjectionEnabled)
        obj = self.mocker.proxy(obj_dummy, spec=None)
        self.mock_extender_values(obj, preLatexCode=None)

        self.replay()
        latex = self.layout.render_latex_for(obj)

        self.assertEqual(latex.strip(), r'\label{path:/path/to/obj}')

    def test_column_layout_injected(self):
        obj1 = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(
            obj1, preferredColumnLayout=NO_PREFERRED_LAYOUT)

        obj1a = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(
            obj1a, preferredColumnLayout=TWOCOLUMN_LAYOUT)

        obj1b = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(
            obj1b, preferredColumnLayout=ONECOLUMN_LAYOUT)

        obj1c = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(
            obj1c, preferredColumnLayout=NO_PREFERRED_LAYOUT)

        obj2 = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(
            obj2, preferredColumnLayout=TWOCOLUMN_LAYOUT)

        self.replay()

        latex_obj1 = self.layout.render_latex_for(obj1)
        latex_obj1a = self.layout.render_latex_for(obj1a)
        latex_obj1b = self.layout.render_latex_for(obj1b)
        latex_obj1c = self.layout.render_latex_for(obj1c)
        latex_obj2 = self.layout.render_latex_for(obj2)

        self.assertNotIn('column', latex_obj1)
        self.assertIn(r'\twocolumn', latex_obj1a)
        self.assertIn(r'\onecolumn', latex_obj1b)
        self.assertNotIn('column', latex_obj1c)
        self.assertIn(r'\twocolumn', latex_obj2)

    def test_latex_clearpage_injected(self):
        normal_obj = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(normal_obj)

        pre_clearpage_obj = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(pre_clearpage_obj, preLatexClearpage=True)

        post_clearpage_obj = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(post_clearpage_obj, postLatexClearpage=True)

        for obj in [normal_obj, pre_clearpage_obj, post_clearpage_obj]:
            self.expect(obj.getPhysicalPath()).result(['', 'myobj'])

        self.replay()

        self.assertNotIn(r'\clearpage',
                         self.layout.render_latex_for(normal_obj))

        self.assertIn(r'\clearpage',
                         self.layout.render_latex_for(pre_clearpage_obj))

        self.assertIn(r'\clearpage',
                         self.layout.render_latex_for(post_clearpage_obj))

    def test_latex_newpage_injected(self):
        normal_obj = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(normal_obj)

        pre_newpage_obj = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(pre_newpage_obj, preLatexNewpage=True)

        for obj in [normal_obj, pre_newpage_obj]:
            self.expect(obj.getPhysicalPath()).result(['', 'myobj'])

        self.replay()

        self.assertNotIn(r'\newpage',
                         self.layout.render_latex_for(normal_obj))

        self.assertIn(r'\newpage',
                         self.layout.render_latex_for(pre_newpage_obj))

    def test_landscape_mode(self):
        obj1 = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(obj1, latexLandscape=False)

        obj2 = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(obj2, latexLandscape=True)

        self.replay()

        latex_obj1 = self.layout.render_latex_for(obj1)
        latex_obj2 = self.layout.render_latex_for(obj2)

        self.assertNotIn(r'landscape', latex_obj1)

        self.assertIn(r'landscape', latex_obj2)
        self.assertIn(r'\begin{landscape}', latex_obj2)
        self.assertIn(r'\end{landscape}', latex_obj2)

    def test_landscape_nested(self):
        foo = self.providing_stub([ILaTeXCodeInjectionEnabled, IFoo])
        self.mock_extender_values(foo, latexLandscape=True)

        bar = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(bar, latexLandscape=False)

        baz = self.providing_stub(ILaTeXCodeInjectionEnabled)
        self.mock_extender_values(baz, latexLandscape=True)

        class FooLaTeXView(object):

            def __init__(self, context, request, layout):
                self.layout = layout

            def render(self):
                return '\n'.join((
                        'Bar: %s' % self.layout.render_latex_for(bar).strip(),
                        'Baz: %s' % self.layout.render_latex_for(baz).strip(),
                        ))

        self.mock_adapter(FooLaTeXView, ILaTeXView,
                          (IFoo, Interface, Interface))

        self.replay()

        self.assertEqual(
            '\n'.join((
                    r'\label{path:/path/to/obj}',
                    r'',
                    r'\begin{landscape}',
                    r'Bar: \label{path:/path/to/obj}',
                    r'Baz: \label{path:/path/to/obj}',
                    r'\end{landscape}')),

            self.layout.render_latex_for(foo))


class TestLaTeXInjectionController(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestLaTeXInjectionController, self).setUp()

        context = self.create_dummy()
        self.request = self.providing_stub(IWithinBookLayer)
        builder = self.create_dummy()

        self.layout = BaseLayout(context, self.request, builder)

    def test_component_is_registered(self):
        self.replay()
        component = queryMultiAdapter((self.layout, self.request),
                                      ILaTeXInjectionController)

        self.assertNotEquals(component, None)

    def test_component_implements_interface(self):
        self.replay()
        component = getMultiAdapter((self.layout, self.request),
                                    ILaTeXInjectionController)
        verifyClass(ILaTeXInjectionController, type(component))

    def test_layout_columns_defaults_to_onecolumn(self):
        self.replay()
        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertEqual(controller.get_current_layout(),
                         ONECOLUMN_LAYOUT)

    def test_layout_columns_layout_switching_is_persistent(self):
        self.replay()

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertEqual(controller.get_current_layout(), ONECOLUMN_LAYOUT)
        controller.set_layout(TWOCOLUMN_LAYOUT)
        self.assertEqual(controller.get_current_layout(), TWOCOLUMN_LAYOUT)

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertEqual(controller.get_current_layout(), TWOCOLUMN_LAYOUT)
        controller.set_layout(ONECOLUMN_LAYOUT)
        self.assertEqual(controller.get_current_layout(), ONECOLUMN_LAYOUT)

    def test_layout_returns_latex_code(self):
        self.replay()

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)

        self.assertEqual(controller.set_layout(NO_PREFERRED_LAYOUT), '')
        self.assertEqual(controller.set_layout(ONECOLUMN_LAYOUT), '')

        self.assertEqual(controller.set_layout(TWOCOLUMN_LAYOUT),
                         r'\twocolumn')
        self.assertEqual(controller.set_layout(TWOCOLUMN_LAYOUT), '')
        self.assertEqual(controller.set_layout(NO_PREFERRED_LAYOUT), '')

        self.assertEqual(controller.set_layout(ONECOLUMN_LAYOUT),
                         r'\onecolumn')
        self.assertEqual(controller.set_layout(ONECOLUMN_LAYOUT), '')
        self.assertEqual(controller.set_layout(NO_PREFERRED_LAYOUT), '')

    def test_landscape_disabled_by_default(self):
        self.replay()
        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertFalse(controller.is_landscape())

    def test_landscape_switching_is_persistent(self):
        self.replay()

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertFalse(controller.is_landscape())

        controller.set_landscape(1, True)
        self.assertTrue(controller.is_landscape())

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertTrue(controller.is_landscape())

        controller.close_landscape(1)
        self.assertFalse(controller.is_landscape())

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertFalse(controller.is_landscape())

    def test_landscape_setting_returns_latex_code(self):
        self.replay()

        controller = getMultiAdapter((self.layout, self.request),
                                     ILaTeXInjectionController)
        self.assertFalse(controller.is_landscape())

        self.assertEqual(controller.set_landscape('a', True),
                         r'\begin{landscape}')

        self.assertEqual(controller.set_landscape('b', True), r'')
        self.assertEqual(controller.set_landscape('c', False), r'')
        self.assertEqual(controller.close_landscape('c'), r'')
        self.assertEqual(controller.close_landscape('b'), r'')
        self.assertEqual(controller.close_landscape('a'), r'\end{landscape}')

    def test_landscape_enabling_requires_package(self):
        layout = self.mocker.proxy(self.layout)
        self.expect(layout.use_package('lscape'))
        self.replay()

        controller = getMultiAdapter((layout, self.request),
                                     ILaTeXInjectionController)
        self.assertFalse(controller.is_landscape())
        controller.set_landscape(1, True)



from ftw.book.interfaces import IBook
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.view import RecursiveLaTeXView
from zope.component import adapts
from zope.interface import Interface


class BookLaTeXView(RecursiveLaTeXView):
    adapts(IBook, Interface, ILaTeXLayout)

    def render(self):
        return self.render_children()



from Products.ATContentTypes.interfaces.link import IATLink
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from zope.component import getMultiAdapter
from zope.interface import alsoProvides


class TestLinkLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestLinkLaTeXView, self).setUp()

        layout_obj = self.create_dummy()
        alsoProvides(layout_obj, ILaTeXLayout)
        self.converter = getMultiAdapter((object(), object(), layout_obj),
                                         IHTML2LaTeXConverter)

    def test_converter(self):
        request = self.create_dummy()
        context = self.providing_stub([IATLink])

        url = 'http://www.google.ch/'
        description = 'a link to google'

        self.expect(context.Title()).result('My link')
        self.expect(context.remoteUrl).result(url)
        self.expect(context.getRawDescription()).result(description)

        layout = self.providing_stub([ILaTeXLayout])
        self.expect(layout.get_converter()).result(self.converter)

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.render()

        self.assertEqual(
            latex,
            '\n'.join((
                    r'\begin{description}',
                    r'\item[My link ' + \
                        '(\href{%s}{%s})]' % (url, url) + \
                        r'{%s}' % description,
                    r'\end{description}')))



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from unittest2 import TestCase
from plone.testing.z2 import Browser
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.portlets.interfaces import IPortletManager
from zope.component import getUtility, getMultiAdapter
from plone.portlets.interfaces import IPortletAssignmentMapping
import transaction

class TestEventhandlersFunctional(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.portal.invokeFactory('Folder', 'folder-1')
        self.portal.invokeFactory('Folder', 'folder-2')
        transaction.commit()

    def create_book(self):
        self.browser.open(self.portal.absolute_url()+'/folder-1/createObject?type_name=Book')
        self.browser.getControl(name="title").value = "Hans Peter"
        self.browser.getControl(name="form.button.save").click()


    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD,))


    def get_portlet(self, obj):
        manager = getUtility(IPortletManager, name='plone.leftcolumn')
        mapping = getMultiAdapter((obj, manager),
                              IPortletAssignmentMapping).__of__(obj)
        return mapping['navigation']

    def test_move_book(self):
        self._auth()
        self.create_book()
        self.browser.getLink("Cut").click()
        self.browser.open(self.portal.absolute_url()+'/folder-2')
        self.browser.getLink("Paste").click()
        self.browser.open(self.browser.url+'/hans-peter')
        obj = self.portal['folder-2']['hans-peter']
        portlet = self.get_portlet(obj)
        self.assertEqual(portlet.root, '/folder-2/hans-peter')

    def test_copy_book(self):
        self._auth()
        self.create_book()
        self.browser.getLink("Copy").click()
        self.browser.open(self.portal.absolute_url()+'/folder-2')
        self.browser.getLink("Paste").click()
        self.browser.open(self.browser.url+'/hans-peter')
        obj = self.portal['folder-2']['hans-peter']
        portlet = self.get_portlet(obj)
        self.assertEqual(portlet.root, '/folder-2/hans-peter')

    def test_after_creation(self):
        self._auth()
        self.create_book()
        obj = self.portal['folder-1']['hans-peter']
        portlet = self.get_portlet(obj)
        self.assertEqual(portlet.root, '/folder-1/hans-peter')



from ftw.book.layer import providing_book_layers
from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from unittest2 import TestCase
from zope.interface import directlyProvidedBy


class TestContextManager(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_context_manager_adds_book_layers(self):
        thelist = list(directlyProvidedBy(self.portal.REQUEST))

        with providing_book_layers(self.portal, self.portal.REQUEST):
            diff = [it for it in list(
                    directlyProvidedBy(self.portal.REQUEST)) if it not in thelist]

            self.assertEquals(
                ['<InterfaceClass ftw.book.interfaces.IWithinBookLayer>'],
                [str(item) for item in diff])

        self.assertEquals(thelist,
                          list(directlyProvidedBy(self.portal.REQUEST)))



from Products.CMFPlone.utils import getFSVersionTuple
from ftw.testing.genericsetup import GenericSetupUninstallMixin
from ftw.testing.genericsetup import apply_generic_setup_layer
from unittest2 import TestCase


@apply_generic_setup_layer
class TestGenericSetupUninstall(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.book'
    additional_products = (
        'simplelayout.base',
        'simplelayout.ui.base',
        'simplelayout.ui.dragndrop',
        'ftw.contentpage',
        )

    @property
    def skip_files(self):
        if getFSVersionTuple() < (4, 3):
            # Plone <= 4.2 does not support removing thins in tinymce.xml
            return ('tinymce.xml')
        else:
            return ()



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from unittest2 import TestCase


class TestBookNavigation(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    @browsing
    def test_portlet_is_crated_when_book_is_crated(self, browser):
        folder = create(Builder('folder').titled('The Folder'))
        book = create(Builder('book').titled('The Book').within(folder))
        create(Builder('chapter').titled('The Chapter').within(book))

        browser.login().visit(book)
        self.assertEquals(
            ['The Book', 'The Chapter'],
            browser.css('.portletNavigationTree .navTreeItem').text)



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from unittest2 import TestCase
from zipfile import ZipFile
from StringIO import StringIO


class TestBookZipexport(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.folder = create(Builder('folder'))

        self.book = create(Builder('book').titled('The Book')
                           .within(self.folder))
        chapter1 = create(Builder('chapter').titled('First Chapter')
                          .within(self.book))

        listingblock1 = create(Builder('listingblock').within(chapter1))
        create(Builder('image').titled('The Image 1').within(listingblock1)
               .with_dummy_content())

        subchapter = create(Builder('chapter').titled('The SubChapter')
                            .within(chapter1))
        create(Builder('book textblock').titled('Hidden Title Block')
               .having(showTitle=False).within(subchapter))
        create(Builder('book textblock').titled('Visible Title Block')
               .having(showTitle=True).within(subchapter))

        listingblock2 = create(Builder('listingblock').within(subchapter))
        create(Builder('image').titled('The Image 2').within(listingblock2)
               .with_dummy_content())

        chapter2 = create(Builder('chapter').titled('Second Chapter')
                          .within(self.book))

        listingblock3 = create(Builder('listingblock').within(chapter2))
        create(Builder('image').titled('The Image 3').within(listingblock3)
               .with_dummy_content())

    @browsing
    def test_zipexport_integration(self, browser):
        browser.login().visit(self.folder, view='zip_export')

        self.assertEquals('application/zip', browser.headers['Content-Type'])

        zipfile = ZipFile(StringIO(browser.contents))
        self.assertEquals(
            ['the-book.pdf',
             'The Book/First Chapter/image.gif',
             'The Book/First Chapter/The SubChapter/image.gif',
             'The Book/Second Chapter/image.gif'],
            zipfile.namelist())



"""Adds pre- and post-latex-views for every object within a book,
mixing in the additional preLatexCode and postLatexCode.
"""

from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import ILaTeXInjectionController
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.interfaces import ONECOLUMN_LAYOUT
from ftw.book.interfaces import TWOCOLUMN_LAYOUT
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.view import MakoLaTeXView
from zope.annotation import IAnnotations
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.interface import implements


class LaTeXInjectionController(object):
    adapts(ILaTeXLayout, IWithinBookLayer)
    implements(ILaTeXInjectionController)

    ANNOTATION_KEY = 'latex-injection-controller'

    def __init__(self, layout, request):
        self.layout = layout
        self.request = request
        self._storage = None

    def get_current_layout(self):
        return self._get_storage().get('current_layout', ONECOLUMN_LAYOUT)

    def set_layout(self, layout):
        if not layout or layout not in (ONECOLUMN_LAYOUT, TWOCOLUMN_LAYOUT):
            return ''

        if layout is self.get_current_layout():
            return ''

        self._get_storage()['current_layout'] = layout
        if layout == ONECOLUMN_LAYOUT:
            return r'\onecolumn'

        elif layout == TWOCOLUMN_LAYOUT:
            return r'\twocolumn'

        else:
            return ''

    def is_landscape(self):
        return self._get_storage().get('landscape_enabled', False)

    def set_landscape(self, obj, enabled):
        if not enabled or self.is_landscape():
            return r''

        self.layout.use_package('lscape')

        storage = self._get_storage()
        storage['landscape_enabled'] = True
        storage['landscape_closing_object'] = obj

        return r'\begin{landscape}'

    def close_landscape(self, obj):
        if not self.is_landscape():
            return r''

        storage = self._get_storage()
        if storage.get('landscape_closing_object', None) == obj:
            storage['landscape_enabled'] = False
            storage['landscape_closing_object'] = None
            return r'\end{landscape}'

        else:
            return r''

    def _get_storage(self):
        if self._storage is None:
            ann = IAnnotations(self.layout)
            key = self.__class__.ANNOTATION_KEY
            if key not in ann:
                ann[key] = {}
            self._storage = ann[key]
        return self._storage


class InjectionLaTeXViewBase(MakoLaTeXView):

    def get_rendered_latex_for(self, fieldname):
        code = self.get_field_value(fieldname)
        if not code:
            return ''

        latex = [
            '',
            '%% ---- LaTeX injection (%s) at %s' % (
                fieldname,
                '/'.join(self.context.getPhysicalPath())),
            code,
            '%% ---- end LaTeX injection (%s)' % fieldname
            ]

        return '\n'.join(latex)

    def get_field_value(self, fieldname):
        """Returns the value of the field ``fieldname`` or ``None``.
        """
        field = self.context.Schema().getField(fieldname)
        if field is None:
            return None

        else:
            return field.get(self.context)

    def get_controller(self):
        return getMultiAdapter((self.layout, self.request),
                               ILaTeXInjectionController)


class PreInjectionLaTeXView(InjectionLaTeXViewBase):
    """Mixes in the preLatexCode for every object providing
    ILaTeXCodeInjectionEnabled and within a book.
    """

    adapts(ILaTeXCodeInjectionEnabled, IWithinBookLayer, Interface)

    def render(self):
        latex = []

        self.layout.use_package('hyperref')
        path = '/'.join(self.context.getPhysicalPath())
        latex.append(r'\label{path:%s}' % path)

        if self.get_field_value('preLatexClearpage'):
            latex.append(r'\clearpage')

        if self.get_field_value('preLatexNewpage'):
            latex.append(r'\newpage')

        latex.append(self._render_preferred_layout())
        latex.append(self._render_landscape())
        latex.append(self.get_rendered_latex_for('preLatexCode'))

        return '\n'.join(latex).strip()

    def _render_landscape(self):
        landscape = self.get_field_value('latexLandscape')
        controller = self.get_controller()
        return controller.set_landscape(self.context, landscape)

    def _render_preferred_layout(self):
        preferred_layout = self.get_field_value('preferredColumnLayout')
        controller = self.get_controller()
        return controller.set_layout(preferred_layout)


class PostInjectionLaTeXView(InjectionLaTeXViewBase):
    """Mixes in the postLatexCode for every object providing
    ILaTeXCodeInjectionEnabled and within a book.
    """

    adapts(ILaTeXCodeInjectionEnabled, IWithinBookLayer, Interface)

    def render(self):
        latex = []

        if self.get_field_value('postLatexClearpage'):
            latex.append(r'\clearpage')

        latex.append(self._render_landscape())
        latex.append(self.get_rendered_latex_for('postLatexCode'))

        return '\n'.join(latex).strip()

    def _render_landscape(self):
        controller = self.get_controller()
        return controller.close_landscape(self.context)



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from ftw.book import _
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IHTMLBlock
from ftw.contentpage.content import textblock
from ftw.contentpage.content.schema import finalize
from zope.interface import implements


if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


htmlblock_schema = ATContentTypeSchema.copy() + \
    textblock.default_schema.copy()

htmlblock_schema['title'].required = False
htmlblock_schema['description'].widget.visible = {'view': 'invisible',
                                                  'edit': 'invisible'}

htmlblock_schema['text'].validators = ()
htmlblock_schema['text'].default_output_type = 'text/html'
htmlblock_schema['text'].widget = atapi.TextAreaWidget(
    label=_(u'label_html', default=u'HTML'),
    description='',
    rows=32,
    cols=70)

finalize(htmlblock_schema)


class HTMLBlock(textblock.TextBlock):
    """HTML block for books.
    """

    security = ClassSecurityInfo()
    implements(IHTMLBlock)
    schema = htmlblock_schema


atapi.registerType(HTMLBlock, PROJECTNAME)



from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from ftw.builder import Builder
from ftw.builder import create
from unittest2 import TestCase
from zope.interface import noLongerProvides


class TestLatexInjectionExtender(TestCase):

    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        self.folder = create(Builder('folder'))
        self.page = create(Builder('page')
                           .titled('First page')
                           .within(self.folder))

    def add_book(self):
        self.book = create(Builder('book')
                           .titled('My Book')
                           .within(self.folder))

        self.chapter = create(Builder('chapter')
                              .titled('Chapter One')
                              .within(self.book))

        self.textblock = create(Builder('book textblock')
                                .titled('TextBlock One')
                                .within(self.chapter))

    def test_page_is_enabled_on_all_objects(self):
        self.add_book()
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.folder))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.page))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.book))
        self.assertTrue(ILaTeXCodeInjectionEnabled.providedBy(self.chapter))

    def test_page_has_no_injected_fields(self):
        self.assertEquals(self.page.Schema().getField('preLatexCode'), None)
        self.assertEquals(self.page.Schema().getField('postLatexCode'), None)

    def test_folder_has_no_injected_fields(self):
        self.assertEquals(self.folder.Schema().getField('preLatexCode'),
                          None)
        self.assertEquals(
            self.folder.Schema().getField('postLatexCode'), None)

    def test_book_has_injected_fields(self):
        self.add_book()
        try:
            schema = self.book.Schema()

            # general book fields
            self.assertTrue(schema.getField('preLatexCode'))
            self.assertTrue(schema.getField('postLatexCode'))
            self.assertTrue(schema.getField('preferredColumnLayout'))
            self.assertFalse(schema.getField('latexLandscape'))
            self.assertFalse(schema.getField('preLatexClearpage'))
            self.assertFalse(schema.getField('postLatexClearpage'))
            self.assertFalse(schema.getField('preLatexNewpage'))
            self.assertFalse(schema.getField('hideFromTOC'))

        finally:
            # cleanup
            noLongerProvides(self.book.REQUEST, IWithinBookLayer)

    def test_chapter_has_injected_fields(self):
        self.add_book()
        try:
            schema = self.chapter.Schema()

            # general book fields
            self.assertTrue(schema.getField('preLatexCode'))
            self.assertTrue(schema.getField('postLatexCode'))
            self.assertTrue(schema.getField('preferredColumnLayout'))
            self.assertTrue(schema.getField('latexLandscape'))
            self.assertTrue(schema.getField('preLatexClearpage'))
            self.assertTrue(schema.getField('postLatexClearpage'))
            self.assertTrue(schema.getField('preLatexNewpage'))
            self.assertFalse(schema.getField('hideFromTOC'))

        finally:
            # cleanup
            noLongerProvides(self.chapter.REQUEST, IWithinBookLayer)

    def test_textblock_has_injected_fields(self):
        self.add_book()
        try:
            schema = self.textblock.Schema()

            # general book fields
            self.assertTrue(schema.getField('preLatexCode'))
            self.assertTrue(schema.getField('postLatexCode'))
            self.assertTrue(schema.getField('preferredColumnLayout'))
            self.assertTrue(schema.getField('latexLandscape'))
            self.assertTrue(schema.getField('preLatexClearpage'))
            self.assertTrue(schema.getField('postLatexClearpage'))
            self.assertTrue(schema.getField('preLatexNewpage'))
            self.assertTrue(schema.getField('hideFromTOC'))

        finally:
            # cleanup
            noLongerProvides(self.textblock.REQUEST, IWithinBookLayer)

    def test_textblock_reordering(self):
        self.add_book()
        try:
            schema = self.textblock.Schema()
            fieldnames = schema.keys()

            # hideFromTOC should be right after showTitle
            self.assertEqual(fieldnames[fieldnames.index('showTitle') + 1],
                             'hideFromTOC')

        finally:
            # cleanup
            noLongerProvides(self.textblock.REQUEST, IWithinBookLayer)



from Products import DataGridField
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from Products.Archetypes.public import DisplayList
from ftw.book import _
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import ITable
from ftw.book.table import generator
from ftw.contentpage.content.schema import finalize
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.interface import implements


try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi


MAX_AMOUNT_OF_COLUMNS = 12
MAX_AMOUNT_OF_HEADER_ROWS = 5
MAX_AMOUNT_OF_FOOTER_ROWS = 5
BORDER_LAYOUTS = (
    ('grid', _(u'table_label_gridLayout', default=u'Grid Layout')),
    ('invisible', _(u'table_label_invisible', default=u'No borders')),
    ('fancy_listing', _(u'table_label_fancy_listing',
        default=u'Horizontal borders')),
    )


table_schema = (ATContentTypeSchema.copy() +
                atapi.Schema((

            atapi.BooleanField(
                name='showTitle',
                schemata='default',
                default=False,
                widget=atapi.BooleanWidget(
                    label=_(u'label_show_title', default=u'Show title'),
                    ),
                ),

            DataGridField.DataGridField(
                name='data',
                schemata='default',
                searchable=True,
                required=False,
                columns=['column_%i' % i for i in
                         range(MAX_AMOUNT_OF_COLUMNS)] + ['row_format'],
                widget=DataGridField.DataGridWidget(
                    label=_(u'label_table_content', default=u'Table content'),
                    macro='datagridwidget_bibliothek_table',
                    columns=dict([
                        ('column_%i' % i, DataGridField.Column(
                            label=_(
                                'Column ${num}',
                                mapping={'num': (i + 1)})))
                        for i in range(MAX_AMOUNT_OF_COLUMNS)] + [(
                            'row_format',
                            DataGridField.SelectColumn(
                                label='Format',
                                vocabulary='getRowFormatVocabulary',
                            ),
                        )]
                        ),
                    ),
                ),

            atapi.TextField(
                name='footnoteText',
                schemata='default',
                required=False,
                searchable=True,
                allowable_content_types=('text/html', ),
                default_content_type='text/html',
                validators=('isTidyHtmlWithCleanup', ),
                default_input_type='text/html',
                default_output_type='text/x-html-safe',
                widget=atapi.RichWidget(
                    label=_(u'label_footnote_text', default=u'Footnote Text'),
                    ),
                ),

            DataGridField.DataGridField(
                name='columnProperties',
                schemata='Layout',
                searchable=False,
                required=False,
                allow_insert=False,
                allow_delete=False,
                allow_reorder=False,
                columns=(
                    'columnId',
                    'columnTitle',
                    'active',
                    'alignment',
                    'bold',
                    'indent',
                    'width',
                    ),
                widget=DataGridField.DataGridWidget(
                    label=_(u'label_column_properties',
                        default=u'Column properties'),
                    columns={
                        'columnId': DataGridField.FixedColumn(
                            'column_id',
                            default='column_x',
                            visible=False,
                            ),
                        'columnTitle': DataGridField.FixedColumn(
                            _(u'table_label_column', default=u'Column'),
                            default='Row X',
                            ),
                        'active': DataGridField.CheckboxColumn(
                            _(u'table_label_active', default=u'Active'),
                            default=False,
                            ),
                        'alignment': DataGridField.SelectColumn(
                            _(u'table_label_alignment', default=u'Alignment'),
                            vocabulary='getAlignmentVocabulary',
                            ),
                        'bold': DataGridField.CheckboxColumn(
                            _(u'table_label_bold', default=u'Bold'),
                            default=False,
                            ),
                        'indent': DataGridField.SelectColumn(
                            _(u'table_label_indent', default=u'Indent'),
                            vocabulary='getIndentVocabulary',
                            ),
                        'width': DataGridField.Column(
                            label=_(u'table_label_width',
                                      default=u'Width (%)'),
                            ),
                        },
                    ),
                fixed_rows=[
                    DataGridField.FixedRow(
                        keyColumn='columnId',
                        initialData={
                            'columnId': 'column_%i' % (i),
                            'columnTitle': _(
                                u'Column ${num}',
                                mapping=dict(num=(i + 1))),
                            'active': False,
                            'alignment': '',
                            'bold': False,
                            'indent': '',
                            'width': '10',
                            },
                        ) for i in range(MAX_AMOUNT_OF_COLUMNS)],
                ),

            atapi.StringField(
                name='headerRows',
                schemata='Layout',
                default='1',
                enforceVocabulary=True,
                vocabulary=[
                    (str(i), _('${num} rows', mapping={'num': str(i)}))
                    for i in range(MAX_AMOUNT_OF_HEADER_ROWS + 1)],
                widget=atapi.SelectionWidget(
                    label=_(u'label_header_rows',
                        default=u'Amount of header rows'),
                    ),
                ),

            atapi.StringField(
                name='footerRows',
                schemata='Layout',
                default=0,
                enforceVocabulary=True,
                vocabulary=[
                    (str(i), _('${num} rows', mapping={'num': str(i)}))
                    for i in range(MAX_AMOUNT_OF_FOOTER_ROWS + 1)],
                widget=atapi.SelectionWidget(
                    label=_(u'label_footer_rows',
                        default=u'Amount of footer rows'),
                    ),
                ),

            atapi.BooleanField(
                name='firstColumnIsHeader',
                schemata='Layout',
                default=False,
                widget=atapi.BooleanWidget(
                    label=_(u'label_first_column_is_header',
                        default=u'First column is a header column'),
                    ),
                ),

            atapi.BooleanField(
                name='footerIsBold',
                schemata='Layout',
                default=True,
                widget=atapi.BooleanWidget(
                    label=_(u'label_footer_is_bold',
                        u'Footer rows are bold'),
                    ),
                ),

            atapi.StringField(
                name='borderLayout',
                schemata='Layout',
                default='fancy_listing',
                enforceVocabulary=True,
                vocabulary=BORDER_LAYOUTS,
                widget=atapi.SelectionWidget(
                    label=_(u'label_border_layout', default=u'Border Layout'),
                    )
                ),

            atapi.BooleanField(
                name='noLifting',
                schemata='Layout',
                default=False,
                widget=atapi.BooleanWidget(
                    label=_(u'label_no_lifting', default=u'No lifting'),
                    description=_(
                        u'description_no_lifting',
                        default=u'When exporting the book as PDF the table '
                        u'will be pulled up if there is no content in the '
                        u'first cell. The aim is to place a preceding title '
                        u'at the same height as the first row of the table. '
                        u'For suppressing this behaviour enable this '
                        u'option.'))),

            )))


finalize(table_schema, hide=['description'])


class Table(ATCTContent, HistoryAwareMixin):
    """A Table for ftw.book
    """
    implements(ITable, ISimpleLayoutBlock)

    portal_type = "Table"
    schema = table_schema

    def getTable(self):
        return generator.TableGenerator(self).render()

    def convert_to_int(self, value):
        """ Converts a value to integer. If its not possible we return the
        unconverted value
        """
        try:
            return int(value)
        except ValueError:
            return value

    def getHeaderRows(self, as_int=False):
        """ We need to calculate with the keys of the headerRows. So we need
        integers. But the vocabulary of a ATField needs strings as keys.
        """
        if as_int:
            return self.convert_to_int(self.headerRows)
        return self.headerRows

    def getFooterRows(self, as_int=False):
        """ We need to calculate with the keys of the footerRsows. So we need
        integers. But the vocabulary of a ATField needs strings as keys.
        """
        if as_int:
            return self.convert_to_int(self.footerRows)
        return self.footerRows

    def getAlignmentVocabulary(self):
        return DisplayList((
                ('', _('automatically')),
                ('left', _('left')),
                ('right', _('right')),
                ('center', _('center')),
                ))

    def getIndentVocabulary(self):
        return DisplayList((
                ('', _('no indent')),
                ('indent2', _('2mm')),
                ('indent10', _('10mm')),
                ))

    def getRowFormatVocabulary(self):
        return DisplayList((
                ('', _('Normal')),
                ('bold', _('Bold')),
                ('grey', _('Grey font')),

                ('indent2', _('2mm indent')),
                ('indent10', _('10mm indent')),
                ('indent2 bold', _('2mm ind. + bold')),
                ('indent10 bold', _('10mm ind. + bold')),
                ('indent2 grey', _('2mm ind. + grey font')),
                ('indent10 grey', _('10mm ind. + grey font')),

                ('noborders', _('Row without line')),
                ('scriptsize', _('Small font')),
                ('fullColspan', _('Strech first line')),
                ))

atapi.registerType(Table, PROJECTNAME)



from ftw.book.interfaces import IRemark, IAddRemarkLayer
from ftw.book.latex.remark import RemarkLaTeXView
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from zope.component import queryMultiAdapter, getMultiAdapter
from zope.interface import alsoProvides
from zope.interface.verify import verifyClass


class TestRemarkLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def setUp(self):
        super(TestRemarkLaTeXView, self).setUp()
        layout_obj = self.create_dummy()
        alsoProvides(layout_obj, ILaTeXLayout)
        self.converter = getMultiAdapter((object(), object(), layout_obj),
                                         IHTML2LaTeXConverter)

    def test_component_is_registered(self):
        context = self.providing_stub([IRemark])
        request = self.providing_stub([IAddRemarkLayer])
        layout = self.providing_stub([ILaTeXLayout])

        self.replay()

        view = queryMultiAdapter((context, request, layout),
                                 ILaTeXView)

        self.assertEquals(type(view), RemarkLaTeXView)

    def test_component_implements_interface(self):
        self.assertTrue(ILaTeXView.implementedBy(RemarkLaTeXView))
        verifyClass(ILaTeXView, RemarkLaTeXView)


    def test_rendering(self):
        context = self.providing_stub([IRemark])
        self.expect(context.Title()).result('title')
        self.expect(context.getText()).result('foo <b>bar</b> baz')

        request = self.providing_stub([IAddRemarkLayer])

        layout = self.providing_stub([ILaTeXLayout])

        self.expect(layout.get_converter()).result(self.converter)
        self.expect(layout.use_package('color')).result(True)

        self.replay()

        view = queryMultiAdapter((context, request, layout),
                                 ILaTeXView)

        self.assertEquals(view.render(),
            '\\definecolor{light-gray}{gray}{0.8}\\fcolorbox{black}' + \
            '{light-gray}{\\parbox[r]{\\textwidth}{{\\bf title}\\\nfoo ' + \
            '\\textbf{bar} baz\n}}'
        )



from ftw.book.interfaces import IBook
from ftw.book.interfaces import IBookTextBlock
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import IHTML2LaTeXConverter
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from mocker import ANY
from simplelayout.base.interfaces import IBlockConfig
from zope.app.component.hooks import setSite
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.interface import alsoProvides


class TestTextBlockLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def get_mocks(self):
        request = self.create_dummy(getPreferredLanguages=lambda: [])
        alsoProvides(request, IWithinBookLayer)
        alsoProvides(request, IUserPreferredLanguages)

        # mock is not yet in replay mode, so we use another layout dummy..
        layout_obj = self.create_dummy(use_package=lambda pkg_name: None)
        alsoProvides(layout_obj, ILaTeXLayout)
        converter = getMultiAdapter((object(), request, layout_obj),
                                    IHTML2LaTeXConverter)

        context = self.providing_stub([IBookTextBlock, IBlockConfig])

        layout = self.providing_mock([ILaTeXLayout])
        self.expect(layout.get_converter()).result(converter).count(0, None)

        return context, request, layout

    def create_image_mocks(self, image_layout, caption, uid,
                           floated=True):
        context, request, layout = self.get_mocks()

        self.site = self.create_dummy(
            REQUEST=request,
            getSiteManager=getSiteManager)
        setSite(self.site)

        image = self.create_dummy(get_size=lambda: 11, data='hello world')
        builder = self.mocker.mock()

        self.expect(layout.get_builder()).result(builder)
        self.expect(builder.add_file('%s_image.jpg' % uid, image.data))

        self.expect(layout.use_package('graphicx'))
        if floated:
            self.expect(layout.use_package('wrapfig'))
            self.expect(layout.use_package('checkheight'))
            self.expect(layout.get_builder()).result(builder)
            self.expect(builder.add_file('checkheight.sty', ANY))
            self.expect(builder.build_directory).result('/tmp')

        self.expect(context.getImage()).result(image).count(1, None)
        self.expect(context.image_layout).result(image_layout)
        self.expect(context.getImageCaption()).result(caption)
        self.expect(context.UID()).result(uid)

        return context, request, layout

    def test_get_image_latex_no_latex_with_no_image(self):
        request = self.create_dummy()
        layout = self.create_dummy()

        context = self.providing_stub([IBookTextBlock])
        self.expect(context.getImage()).result(None)

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_image_latex(True)
        self.assertEqual(latex, '')

    def test_get_image_latex_with_layout_noimage(self):
        request = self.create_dummy()
        layout = self.create_dummy()

        context = self.providing_stub([IBookTextBlock, IBlockConfig])

        self.expect(context.getImage()).result(self.create_dummy(
                get_size=lambda: 1))
        self.expect(context.image_layout).result('no-image')
        self.expect(context.getImageCaption()).result('')

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_image_latex(True)
        self.assertEqual(latex, '')

    def test_get_text_latex_some_text(self):
        context, request, layout = self.get_mocks()

        self.expect(context.getText()).result(
            ' Hello little world\n')

        self.replay()
        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_text_latex()

        self.assertEqual(latex, 'Hello little world\n')

    def test_get_text_latex_no_text(self):
        context, request, layout = self.get_mocks()

        self.expect(context.getText()).result('')

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_text_latex()

        self.assertEqual(latex, '')

    def test_full_latex_rendering(self):
        block, request, layout = self.create_image_mocks(
            'small', 'THE image', '123')

        self.expect(block.getShowTitle()).result(True)
        self.expect(block.pretty_title_or_id()).result(
            'My <b>block</b> title')
        self.expect(block.getText()).result(
            'Thats <b>some</b> text.   ')

        book = self.providing_stub([IBook])
        self.set_parent(block, book)

        schema = self.stub()
        self.expect(block.Schema()).result(schema)
        self.expect(schema.getField('hideFromTOC').get(block)
                    ).result(False)

        self.replay()

        view = getMultiAdapter((block, request, layout), ILaTeXView)
        latex = view.render()

        self.assertIn(r'\chapter{My \textbf{block} title}', latex)
        self.assertIn(r'\end{wrapfigure}', latex)
        imagepart, textpart = latex.split(r'\end{wrapfigure}')

        self.assertIn(r'\includegraphics', imagepart)
        self.assertIn('123_image', imagepart)
        self.assertIn(r'Thats \textbf{some} text.', textpart)

    def test_heading_converts_to_bold(self):
        # Within books the book structure (section, subsection, etc) is
        # defined using chapters and block titles. Therefore the
        # text of the block should not contain headings since it would
        # result in inconsistent chapter numberings and other problems.

        context, request, layout = self.get_mocks()

        self.expect(context.getText()).result(
            '<h1>the</h1> <h2>heading</h2> <h3>tags</h3> <h4>will</h4> ' + \
                '<h5>be</h5> <h5>bold</h5>.')

        self.replay()
        view = getMultiAdapter((context, request, layout), ILaTeXView)
        latex = view.get_text_latex()

        self.assertEqual(
            latex,
            '{\\bf the} {\\bf heading} {\\bf tags} {\\bf will} ' + \
                '{\\bf be} {\\bf bold}.\n')

    def test_full_latex_rendering_not_floatable(self):
        """Using "full" layout should not make a floatable image
        (wrapfigure), even there is also text in the block.
        """
        block, request, layout = self.create_image_mocks(
            'full', 'THE image', '123', floated=False)

        self.expect(block.getShowTitle()).result(False)
        self.expect(block.getText()).result(
            'Text')

        book = self.providing_stub([IBook])
        self.set_parent(block, book)

        self.replay()

        view = getMultiAdapter((block, request, layout), ILaTeXView)
        latex = view.render()

        self.assertNotIn(r'\end{wrapfigure}', latex)
        self.assertIn(r'Text', latex)
        self.assertIn(r'\includegraphics', latex)



from ftw.book.interfaces import IBook
from ftw.book.latex.book import BookLaTeXView
from ftw.book.testing import LATEX_ZCML_LAYER
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.testing import MockTestCase
from zope.component import getMultiAdapter


class TestBookLaTeXView(MockTestCase):

    layer = LATEX_ZCML_LAYER

    def test_returns_rendered_children_latex(self):
        context = self.providing_mock([IBook])
        request = self.mocker.mock()
        layout = self.providing_mock([ILaTeXLayout])

        self.expect(layout.context).result(context)
        self.expect(request.get('paths', None)).result(None)

        child1 = self.mocker.mock()
        child2 = self.mocker.mock()
        self.expect(context.listFolderContents()).result([child1, child2])

        self.expect(layout.render_latex_for(child1)).result(
            'child one latex')
        self.expect(layout.render_latex_for(child2)).result(
            'child two latex')

        self.replay()

        view = getMultiAdapter((context, request, layout), ILaTeXView)
        self.assertTrue(isinstance(view, BookLaTeXView))

        self.assertEqual(
            view.render(),
            'child one latex\nchild two latex')



from ftw.book.helpers import BookHelper
from ftw.book.interfaces import IChapter
from ftw.book.latex import utils
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.view import RecursiveLaTeXView
from zope.component import adapts
from zope.interface import Interface


class ChapterLaTeXView(RecursiveLaTeXView):
    adapts(IChapter, Interface, ILaTeXLayout)

    def render(self):
        latex = self.get_heading_counters_latex()
        latex += utils.get_latex_heading(self.context, self.layout)
        latex += self.render_children()
        return latex

    def get_heading_counters_latex(self):
        if self.context != self.layout.context:
            # Only set the heading counters when exporting this chapter
            # directly. Otherwise it is not the first content.
            return ''

        counters = (
            'chapter',
            'section',
            'subsection',
            'subsubsection',
            'paragraph',
            'subparagraph',
            )

        helper = BookHelper()
        heading_numbers = helper.get_chapter_level(self.context)
        latex = []

        if heading_numbers[-1] == 1:
            del heading_numbers[-1]
        else:
            heading_numbers[-1] -= 1

        for level, num in enumerate(heading_numbers):
            latex.append(r'\setcounter{%s}{%s}' % (
                    counters[level], num))

        if latex:
            latex.append('')

        return '\n'.join(latex)



from ftw.book.latex.defaultlayout import IDefaultBookLayoutSelectionLayer
from ftw.book.latex.listingblock import add_table_column_widths
from ftw.book.latex.listingblock import remove_html_links
from ftw.book.latex.listingblock import remove_table_summary
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.pdfgenerator.interfaces import IPDFAssembler
from unittest2 import TestCase
from zope.component import getMultiAdapter


class TestRemoveHTMLLinks(TestCase):

    def test_removes_links_from_html(self):
        html = 'foo <a /> bar'
        self.assertEquals('foo  bar', remove_html_links(html))

    def test_link_containments_are_kept(self):
        html = '<a>foo</a> bar'
        self.assertEquals('foo bar', remove_html_links(html))


class TestRemoveTableSummary(TestCase):

    def test_removes_table_summary_attribtue(self):
        html = '<table class="listing" summary="Foo"></table>'
        self.assertEquals('<table class="listing"></table>',
                          remove_table_summary(html))


class TestAddTableColumnWidths(TestCase):

    def test_adds_width_to_columns(self):
        input = '\n'.join((
                '<table><colgroup>',
                '<col class="col-modified" />',
                '<col class="col-Creator" />',
                '<col class="col-getObjSize" />',
                '<col />',
                '</colgroup></table>'))

        expected = ''.join((
                '<table><colgroup>',
                '<col class="col-modified" width="12%">',
                '<col class="col-Creator" width="15%">',
                '<col class="col-getObjSize" width="10%">',
                '<col width="63%">',
                '</colgroup></table>'))

        self.assertEquals(
            expected,
            add_table_column_widths(input).replace('\n', ''))


class TestListingBlockLaTeXView(TestCase):

    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        self.book = create(Builder('book')
                           .with_layout(IDefaultBookLayoutSelectionLayer))
        self.chapter = create(Builder('chapter').within(self.book))

    def render_latex(self, obj):
        assembler = getMultiAdapter((self.book, obj.REQUEST), IPDFAssembler)
        view = getMultiAdapter((obj, obj.REQUEST, assembler.get_layout()),
                               ILaTeXView)
        return view.render()

    def test_title_is_shown_when_showTitle_enabled(self):
        listingblock = create(Builder('listingblock')
                              .titled('Files')
                              .having(showTitle=True)
                              .within(self.chapter))

        self.assertIn(r'\section{Files}', self.render_latex(listingblock))

    def test_title_is_not_shown_showTitle_disabled(self):
        listingblock = create(Builder('listingblock')
                              .titled('Files')
                              .having(showTitle=False)
                              .within(self.chapter))

        self.assertNotIn(r'Files', self.render_latex(listingblock))

    def test_title_not_in_table_of_contents(self):
        listingblock = create(Builder('listingblock')
                              .titled('Files')
                              .having(showTitle=True,
                                      hideFromTOC=True)
                              .within(self.chapter))

        self.assertIn(r'\section*{Files}', self.render_latex(listingblock))

    def test_table_lists_contents(self):
        listingblock = create(Builder('listingblock')
                              .having(showTitle=False)
                              .within(self.chapter))
        create(Builder('file').titled('The File').within(listingblock))

        latex = self.render_latex(listingblock)
        self.assertIn(r'\begin{tabular}', latex)
        self.assertIn(r'{The File}', latex)



from Products.CMFCore.utils import getToolByName
from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from unittest2 import TestCase

TOP = set(['border-top'])
BOTTOM = set(['border-bottom'])
LEFT = set(['border-left'])
RIGHT = set(['border-right'])
ALL = TOP | BOTTOM | LEFT | RIGHT
NONE = set()


def table_borders():
    table = browser.css('table').first
    cell = lambda cell: (cell.text, set(cell.classes) & ALL)
    row = lambda row: map(cell, row.cells)
    return map(row, table.rows)


class TestTableLayouts(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        portal_types = getToolByName(self.layer['portal'], 'portal_types')
        table_fti = portal_types['Table']
        table_fti.global_allow = True

    @browsing
    def test_grid_layout(self, browser):
        table = create(Builder('table')
                       .with_table((('Foo', 'Bar'),
                                    ('1', '2')))
                       .having(borderLayout='grid'))

        browser.login().visit(table, view='block_view')

        self.assertEquals(
            [[('Foo', ALL),
              ('Bar', ALL)],
             [('1', ALL),
              ('2', ALL)]],
            table_borders())

    @browsing
    def test_invisible_layout(self, browser):
        table = create(Builder('table')
                       .with_table((('Foo', 'Bar'),
                                    ('1', '2')))
                       .having(borderLayout='invisible'))

        browser.login().visit(table, view='block_view')

        self.assertEquals(
            [[('Foo', NONE),
              ('Bar', NONE)],
             [('1', NONE),
              ('2', NONE)]],
            table_borders())

    @browsing
    def test_fancy_listing_layout(self, browser):
        table = create(Builder('table')
                       .with_table((('Foo', 'Bar'),
                                    ('1', '2')))
                       .having(borderLayout='fancy_listing'))

        browser.login().visit(table, view='block_view')

        self.assertEquals(
            [[('Foo',  BOTTOM),
              ('Bar', BOTTOM)],
             [('1', BOTTOM),
              ('2', BOTTOM)]],
            table_borders())



from ftw.book.interfaces import ITable
from ftw.pdfgenerator.view import MakoLaTeXView
from zope.component import adapts
from zope.interface import Interface


class TableLaTeXView(MakoLaTeXView):
    adapts(ITable, Interface, Interface)

    def render(self):

        latex = []

        table = self.context.getTable()
        if len(table) > 0:
            latex.append(self.convert(table))

        footnote_text = self.context.getFootnoteText().strip()
        if len(footnote_text) > 0:

            latex.append(r'\vspace{0pt}')
            latex.append('{\\footnotesize %s}' % (
                    self.convert(footnote_text.strip())))

        return '\n'.join(latex)



from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.book.interfaces import IBook
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer


@implementer(IBook)
@adapter(Interface)
def find_book(obj):
    return IBook(aq_parent(aq_inner(obj)))



from Acquisition import aq_inner, aq_parent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.book.browser.reader.interfaces import IBookReaderRenderer
from ftw.book.browser.reader.utils import flaten_tree
from ftw.book.browser.toc_tree import BookTocTree
from ftw.book.interfaces import IBook
from ftw.contentpage.interfaces import IListingBlock
from json import dumps
from plone.app.layout.navigation.navtree import buildFolderTree
from zope.component import queryMultiAdapter
from zope.publisher.browser import BrowserView


RENDER_BLOCKS_PER_REQUEST_THRESHOLD = 4
_marker = object()


class ReaderView(BrowserView):

    template = ViewPageTemplateFile('templates/reader.pt')
    navigation_recurse = ViewPageTemplateFile(
        'templates/navigation_recurse.pt')

    def __init__(self, *args, **kwargs):
        super(ReaderView, self).__init__(*args, **kwargs)
        self._book = None
        self._tree = None

    def __call__(self):
        self.request.set('disable_border', True)
        self.request.set('disable_plone.leftcolumn', True)
        self.request.set('disable_plone.rightcolumn', True)

        return self.template()

    @property
    def book(self):
        if self._book is None:
            self._book = self.get_book_obj()
        return self._book

    @property
    def tree(self):
        if self._tree is None:
            self._tree = self.get_tree(self.book)
        return self._tree

    def render_next(self, block_render_threshold=_marker):
        """Renders the next blocks.
        """
        if block_render_threshold == _marker:
            block_render_threshold = RENDER_BLOCKS_PER_REQUEST_THRESHOLD

        after_uid = self.request.get('after_uid', '')
        loaded_uids = self.request.get('loaded_blocks[]', [])
        insert_after = after_uid or 'TOP'

        data = []

        brainlist = flaten_tree(self.tree)

        found = False

        for brain in brainlist:
            if not found and after_uid == '' and \
                    brain.UID == self.context.UID():
                found = True

            if not found and brain.UID == after_uid:
                found = True
                continue

            elif not found:
                continue

            elif found and brain.UID in loaded_uids:
                # we already have this block loaded
                break

            block_html = self.render_block(brain)
            if not block_html:
                continue

            data.append([brain.UID, block_html])
            block_render_threshold -= 1
            if block_render_threshold == 0:
                break

        response_data = {'insert_after': insert_after,
                         'data': data,
                         'first_uid': data and data[0][0],
                         'last_uid': data and data[-1][0]}
        self.request.response.setHeader('Content-Type', 'application/json')
        return dumps(response_data)

    def render_previous(self, block_render_threshold=_marker):
        """Render previous blocks.
        """
        if block_render_threshold == _marker:
            block_render_threshold = RENDER_BLOCKS_PER_REQUEST_THRESHOLD

        before_uid = self.request.get('before_uid', '')
        loaded_uids = self.request.get('loaded_blocks[]', [])
        insert_before = before_uid

        if before_uid == '' or not loaded_uids:
            # render_next should be called before render_previous, so there
            # should already content be loaded.
            return '{}'

        data = []

        brainlist = flaten_tree(self.tree)
        previous = []
        found = False

        for brain in brainlist:
            if brain.UID == before_uid:
                found = True
                break
            previous.insert(0, brain)

        if not found:
            return u'{}'

        while block_render_threshold > 0 and len(previous) > 0:
            brain = previous.pop(0)

            if brain.UID in loaded_uids:
                break

            block_html = self.render_block(brain)
            if not block_html:
                continue

            data.insert(0, [brain.UID, block_html])
            block_render_threshold -= 1

        response_data = {'insert_before': insert_before,
                         'data': data,
                         'first_uid': data and data[0][0],
                         'last_uid': data and data[-1][0]}
        self.request.response.setHeader('Content-Type', 'application/json')
        return dumps(response_data)

    def render_block(self, brain):
        obj = brain.getObject()
        if IListingBlock.providedBy(aq_parent(aq_inner(obj))):
            return ''

        renderer = queryMultiAdapter((obj, self.request, self),
                                     IBookReaderRenderer)
        if not renderer:
            return ''

        else:
            return renderer.render()

    def get_tree(self, book):
        """Returns an unlimited, recursive navtree of the book.
        """
        query = {
            'path': '/'.join(book.getPhysicalPath())}

        portal_catalog = getToolByName(self.context, 'portal_catalog')

        types = list(portal_catalog.uniqueValuesFor('portal_type'))
        if 'Discussion Item' in types:
            types.remove('Discussion Item')
        query['portal_type'] = types

        return buildFolderTree(book, obj=book, query=query)

    def get_book_obj(self):
        obj = self.context

        while obj is not None:
            if IBook.providedBy(obj):
                return obj

            elif IPloneSiteRoot.providedBy(obj):
                raise Exception('Could not find book.')

            else:
                obj = aq_parent(aq_inner(obj))

        raise Exception('Could not find book.')

    def render_navigation(self, item=None):
        if item is None:
            toc_tree = BookTocTree()
            item = toc_tree(self.tree)

        toc_title = item.get('item').Title

        if item.get('toc_number', None):
            toc_title = '%s %s' % (item.get('toc_number'), toc_title)

        return self.navigation_recurse(**{
                'item': item,
                'toc_title': toc_title,
                'children_ul_class': 'book-reader-navigation-%i' % (
                    item['depth'] + 1)})



from AccessControl.SecurityInfo import ClassSecurityInformation
from Acquisition import aq_inner
from Acquisition import aq_parent
from archetypes.schemaextender.extender import CACHE_KEY
from ftw.book.interfaces import IBook
from ftw.book.interfaces import IWithinBookLayer
from ftw.publisher.core import getLogger
from ftw.publisher.core.interfaces import IDataCollector
from Products.CMFCore.interfaces import ISiteRoot
from zope.dottedname.resolve import resolve
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides
from zope.interface import implements


def dottedname(iface):
    """Returns the dottedname of an interface as string.
    """
    return '.'.join((iface.__module__, iface.__name__))


class BookLayoutRequestLayerCollector(object):
    """IDataCollector for supporting publishing books (ftw.publisher) with
    custom layouts and schema extender fields bound on layout request layer.

    When a custom layout is activated in the book, the request provides the
    layout interface as soon as traversing the book.
    The layout may add schema extender fields when the request provides the
    layout interface.
    In order to publish those field values correctly (with the standard
    FieldData collector), we need to provide the request layer before
    extracting data (sender) or setting data (receiver).

    Schema extender fields may be applied to not only the book itself but also
    to any content within the book. Therfore this collector should be used
    for all objects.

    The collector may be triggered when the book is traversed but also when
    it wasn't, therefore we cannot rely on the default book traversal adapter
    applying the layer interfaces to the request.
    """

    implements(IDataCollector)
    logger = getLogger()
    security = ClassSecurityInformation()

    def __init__(self, context):
        self.context = context

    security.declarePrivate('getData')
    def getData(self):
        book = self.get_parent_book()
        if not book:
            return None

        layers = [IWithinBookLayer]
        layout_layer_name = getattr(book, 'latex_layout', '')
        if layout_layer_name:
            layers.append(resolve(layout_layer_name))

        self.provide_request_layers(layers)
        self.flush_schemaextender_cache()
        return map(dottedname, layers)

    security.declarePrivate('setData')
    def setData(self, layers_dottednames, metadata):
        if not layers_dottednames:
            return

        self.logger.info(
            'BookLayoutRequestLayerCollector: provide request layers {0}'.format(
                layers_dottednames))

        layers = map(resolve, layers_dottednames)
        self.provide_request_layers(layers)
        self.flush_schemaextender_cache()

    def get_parent_book(self):
        context = self.context

        while context and not ISiteRoot.providedBy(context):
            if IBook.providedBy(context):
                return context
            context = aq_parent(aq_inner(context))

        return None

    def provide_request_layers(self, layers):
        """ Add a layer interface on the request
        """
        request = self.context.REQUEST
        layers = [iface for iface in layers if not iface.providedBy(request)]
        ifaces = layers + list(directlyProvidedBy(request))

        # Since we allow multiple markers here, we can't use
        # zope.publisher.browser.applySkin() since this filters out
        # IBrowserSkinType interfaces, nor can we use alsoProvides(), since
        # this appends the interface (in which case we end up *after* the
        # default Plone/CMF skin)
        directlyProvides(request, *ifaces)

    def flush_schemaextender_cache(self):
        """Flushes the schemaextender cache after conditions for
        schema extenders may have changed.
        """
        try:
            delattr(self.context.REQUEST, CACHE_KEY)
        except AttributeError:
            pass



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IBookTextBlock
from ftw.contentpage.content import textblock
from zope.interface import implements


if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


textblock_schema = textblock.textblock_schema.copy()


class BookTextBlock(textblock.TextBlock):
    """Textblock for books.
    """

    security = ClassSecurityInfo()
    implements(IBookTextBlock)
    schema = textblock_schema


atapi.registerType(BookTextBlock, PROJECTNAME)



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content.schemata import NextPreviousAwareSchema
try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from ftw.book import _
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IBook
from zope.interface import implements
from zope.schema.vocabulary import getVocabularyRegistry


BookSchema = (folder.ATFolderSchema.copy() +
              NextPreviousAwareSchema.copy() +
              atapi.Schema((

            atapi.StringField(
                name='latex_layout',
                required=True,

                vocabulary_factory='ftw.book.layoutsVocabulary',
                default_method='getDefaultLaTeXLayout',

                widget=atapi.SelectionWidget(
                    label=_(u'book_label_layout',
                            default=u'Layout'))),

            atapi.BooleanField(
                name='use_titlepage',
                default=True,

                widget=atapi.BooleanWidget(
                    label=_(u'book_label_use_titlepage',
                            default=u'Embedd a title page'),
                    description=_(u'book_help_use_titlepage',
                                  default=u''))),

            atapi.BooleanField(
                name='use_toc',
                default=True,

                widget=atapi.BooleanWidget(
                    label=_(u'book_label_use_toc',
                            default=u'Embedd table of contents'),
                    description=_(u'book_help_use_toc',
                                  default=u''))),

            atapi.BooleanField(
                name='use_lot',
                default=True,

                widget=atapi.BooleanWidget(
                    label=_(u'book_label_use_lot',
                            default=u'Embedd list of tables.'),
                    description=_(u'book_help_use_lot',
                                  default=u''))),

            atapi.BooleanField(
                name='use_loi',
                default=True,

                widget=atapi.BooleanWidget(
                    label=_(u'book_label_use_loi',
                            default=u'Embedd list of illustrations'),
                    description=_(u'book_help_use_loi',
                                  default=u''))),

            atapi.BooleanField(
                name='use_index',
                default=False,

                widget=atapi.BooleanWidget(
                    label=_(u'book_label_use_index',
                            default=u'Embedd subject index')),
                description=_(u'book_help_use_index',
                              default=u'When enabled, a keyword index '
                              'will be included in the PDF.')),

            )))


schemata.finalizeATCTSchema(BookSchema, folderish=True, moveDiscussion=False)


class Book(folder.ATFolder):
    implements(IBook)
    security = ClassSecurityInfo()

    meta_type = "Book"
    schema = BookSchema

    def getDefaultLaTeXLayout(self):
        voc = getVocabularyRegistry().get(self, 'ftw.book.layoutsVocabulary')

        if len(voc) > 0:
            return voc.by_value.keys()[0]
        else:
            return None

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False


atapi.registerType(Book, PROJECTNAME)



from ftw.book.portlets import gotoparent
from plone.app.portlets.portlets import navigation
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getUtility, getMultiAdapter
from zope.component.interfaces import ComponentLookupError
import logging
from Products.CMFCore.utils import getToolByName

LOG = logging.getLogger('ftw.book')


def add_navigation_portlet(object_, event):
    """Adds a custom navigation Portlet for Buch
    """
    if 'portal_factory' in object_.getPhysicalPath():
        # do not run in portal_factory
        pass
    else:
        right_slot_portlets(object_)
        left_slot_portlets(object_)


def right_slot_portlets(object_):
    """ disable portlet inheritance at rightcolumn
    """

    try:
        manager = getUtility(IPortletManager, name='plone.rightcolumn')

    except ComponentLookupError:
        # This happens when the plone site is removed.
        # In this case persistent utilites are already gone.
        # Reindexing is not necessary in this situation, since
        # everything will be gone.
        LOG.error('eventhandler.left_slot_portlets threw', exc_info=True)
        return

    assignable = getMultiAdapter((object_, manager,),
                                 ILocalPortletAssignmentManager)
    assignable.setBlacklistStatus(CONTEXT_CATEGORY, True)


def left_slot_portlets(object_):
    """ disable portlet inheritance at leftcolumn and
    add new navigation portlet
    """

    try:
        manager = getUtility(IPortletManager, name='plone.leftcolumn')

    except ComponentLookupError:
        # This happens when the plone site is removed.
        # In this case persistent utilites are already gone.
        # Reindexing is not necessary in this situation, since
        # everything will be gone.
        LOG.error('eventhandler.left_slot_portlets threw', exc_info=True)
        return

    assignable = getMultiAdapter((object_, manager,),
                                 ILocalPortletAssignmentManager)
    assignable.setBlacklistStatus(CONTEXT_CATEGORY, True)
    mapping = getMultiAdapter((object_, manager),
                              IPortletAssignmentMapping).__of__(object_)

    if 'navigation' in mapping.keys():
        del mapping['navigation']

    if 'go-to-parent-portlet' not in mapping.keys():
        mapping['go-to-parent-portlet'] = gotoparent.Assignment()

    portal_url = getToolByName(object_, 'portal_url')
    relative_path = '/'.join(object_.getPhysicalPath())[
        len(portal_url.getPortalPath()):]

    mapping['navigation'] = navigation.Assignment(
        root=relative_path,
        topLevel=0,
        includeTop=1)



from ftw.book.browser.reader.interfaces import IBookReaderRenderer
from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
from unittest2 import TestCase
from zope.component import getMultiAdapter


class TestBookRenderer(TestCase):
    layer = FTW_BOOK_FUNCTIONAL_TESTING

    @browsing
    def test_book_title_is_rendered(self, browser):
        book = create(Builder('book').titled('The Book'))
        reader_view = book.restrictedTraverse('@@book_reader_view')
        renderer = getMultiAdapter((book, book.REQUEST, reader_view),
                                   IBookReaderRenderer)
        browser.open_html(renderer.render())
        self.assertEquals('The Book', browser.css('h1').first.text)


class TestBlockRenderer(TestCase):
    layer = FTW_BOOK_FUNCTIONAL_TESTING

    @browsing
    def test_book_title_is_rendered(self, browser):
        book = create(Builder('book'))
        chapter = create(Builder('chapter').within(book))
        block = create(Builder('book textblock')
                       .within(chapter)
                       .having(text='<p>Some Text</p>'))

        reader_view = book.restrictedTraverse('@@book_reader_view')
        renderer = getMultiAdapter((block, book.REQUEST, reader_view),
                                   IBookReaderRenderer)
        browser.open_html(renderer.render())
        self.assertEquals('Some Text', browser.css('p').first.text)

    @browsing
    def test_book_internal_links_are_marked_with_class(self, browser):
        book = create(Builder('book').titled('book'))
        chapter = create(Builder('chapter').within(book).titled('chapter'))

        html = '<a class="internal-link"' + \
            ' href="resolveuid/%s">' % IUUID(chapter) + \
            'The Chapter</a>'
        block = create(Builder('book textblock')
                       .within(chapter)
                       .having(text=html))

        reader_view = book.restrictedTraverse('@@book_reader_view')
        renderer = getMultiAdapter((block, book.REQUEST, reader_view),
                                   IBookReaderRenderer)
        browser.open_html(renderer.render())

        link = browser.find('The Chapter')
        self.assertIn('book-internal', link.classes)
        self.assertDictContainsSubset(
            {'data-uid': IUUID(chapter)},
            link.attrib)

    @browsing
    def test_links_to_itself_are_not_marked_as_book_internal(self, browser):
        # Some titles are linked to the default view of the context
        # so that we can exit the reader and edit things.
        # Theese links should not be changed to pointing to the reader view.
        # One example is the title of chapters.

        book = create(Builder('book').titled('book'))
        chapter = create(Builder('chapter')
                         .within(book)
                         .titled('The Chapter'))

        reader_view = book.restrictedTraverse('@@book_reader_view')
        renderer = getMultiAdapter((chapter, book.REQUEST, reader_view),
                                   IBookReaderRenderer)
        browser.open_html(renderer.render())

        link = browser.find('1 The Chapter')
        self.assertNotIn('book-internal', link.classes)



from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from ftw.book.config import PROJECTNAME
from ftw.book.interfaces import IRemark
from ftw.contentpage.content import textblock
from ftw.contentpage.content.schema import finalize
from zope.interface import implements


try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi


remark_schema = ATContentTypeSchema.copy() + \
    textblock.default_schema.copy()
remark_schema['title'].required = False
remark_schema['title'].searchable = 0
finalize(remark_schema, hide=['description', 'showTitle'])


class Remark(textblock.TextBlock):
    """A simplelayout block used for comments
    """

    security = ClassSecurityInfo()
    implements(IRemark)
    schema = remark_schema


atapi.registerType(Remark, PROJECTNAME)



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from unittest2 import TestCase


class TestTable(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.book = create(Builder('book'))
        self.chapter = create(Builder('chapter').within(self.book))

    @browsing
    def test_creating_table(self, browser):
        browser.login().visit(self.chapter)
        factoriesmenu.add('Table')
        browser.fill({
                'Title': 'The Table',
                'Show title': True,
                'columnProperties.active.1': True,
                'columnProperties.active.2': True}).submit()

        self.assertEquals(self.chapter.absolute_url() + '/#the-table',
                          browser.url)
        blocks = browser.css('.BlockOverallWrapper.table')
        self.assertEquals(1, len(blocks),
                          'Expected chapter to have exactly one block')
        block, = blocks

        self.assertEquals('The table has no content yet.'
                          ' Edit the table block for adding content.',
                          block.css('.portalMessage dd').first.text)

    @browsing
    def test_table_rendering(self, browser):
        create(Builder('table')
               .titled('The Table')
               .having(showTitle=True)
               .with_table([['Foo', 'Bar'],
                            ['1', '2'],
                            ['3', '4']])
               .within(self.chapter))

        browser.login().visit(self.chapter)
        block, = browser.css('.BlockOverallWrapper.table')

        self.assertEquals(
            [{'Foo': '1',
              'Bar': '2'},
             {'Foo': '3',
              'Bar': '4'}],
            block.css('table').first.dicts())

    @browsing
    def test_showing_block_title(self, browser):
        create(Builder('table')
               .titled('Visible')
               .with_dummy_table()
               .having(showTitle=True)
               .within(self.chapter))

        create(Builder('table')
               .titled('Hidden')
               .with_dummy_table()
               .having(showTitle=False)
               .within(self.chapter))

        browser.login().visit(self.chapter)
        visible, hidden = browser.css('.BlockOverallWrapper.table')

        self.assertEquals(
            ['Visible'],
            visible.css('table caption').text,
            'Expected block title of the "Visible" block to be visible.')

        self.assertEquals(
            [],
            hidden.css('table caption').text,
            'Expected block title of the "Hidden" block to be hidden.')

    @browsing
    def test_hidden_fields(self, browser):
        browser.login().open(self.chapter)
        factoriesmenu.add('Table')

        self.assertFalse(
            browser.find('Description'),
            '"Description" field should not be visible.')

        self.assertFalse(
            browser.find('Hide from table of contents'),
            '"Hide from table of contents" field should not be visible.')

        self.assertFalse(
            browser.find('Exclude from navigation'),
            '"Exclude from navigation" field should not be visible.')

    @browsing
    def test_latex_fields_available(self, browser):
        browser.login().open(self.chapter)
        factoriesmenu.add('Table')

        form = browser.find('Title').parent('form')
        labels = form.field_labels

        self.assertIn('LaTeX code above content', labels)
        self.assertIn('LaTeX code beneath content', labels)



from ftw.book.interfaces import IBook
from ftw.book.layer import providing_book_layers
from ftw.pdfgenerator.interfaces import IPDFAssembler
from ftw.zipexport.interfaces import IZipRepresentation
from ftw.zipexport.representations.archetypes import FolderZipRepresentation
from Products.CMFPlone.utils import safe_unicode
from StringIO import StringIO
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.interface import Interface


class BookZipRepresentation(FolderZipRepresentation):
    implements(IZipRepresentation)
    adapts(IBook, Interface)

    def get_files(self, path_prefix=u"", recursive=True, toplevel=True):
        filename = '{0}.pdf'.format(self.context.getId())

        with providing_book_layers(self.context, self.request):
            assembler = getMultiAdapter((self.context, self.request),
                                        IPDFAssembler)

            yield (u'{0}/{1}'.format(safe_unicode(path_prefix),
                                     safe_unicode(filename)),
                   StringIO(assembler.build_pdf()))

            folder_contents = super(BookZipRepresentation, self).get_files(
                path_prefix, recursive, toplevel)

            for item in folder_contents:
                yield item



from ftw.book.latex.defaultlayout import IDefaultBookLayoutSelectionLayer
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.pdfgenerator.interfaces import ILaTeXView
from ftw.pdfgenerator.interfaces import IPDFAssembler
from plone.uuid.interfaces import IUUID
from unittest2 import TestCase
from zope.component import getMultiAdapter


class TestBookInternalHyperlinksLaTeX(TestCase):
    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        self.book = create(Builder('book')
                           .titled('the book')
                           .with_layout(IDefaultBookLayoutSelectionLayer))

    def test_book_internal_hyperlinks_have_internal_reference(self):
        chapter = create(Builder('chapter')
                         .within(self.book)
                         .titled('the chapter'))

        html = 'Link to ' + \
            '<a class="internal-link"' + \
            ' href="resolveuid/%s">' % IUUID(chapter) + \
            'The Chapter</a>' + \
            '!'
        block = create(Builder('book textblock')
                       .within(chapter)
                       .having(text=html))

        request = block.REQUEST
        assembler = getMultiAdapter((self.book, request), IPDFAssembler)
        latex_view = getMultiAdapter((block, request, assembler.get_layout()),
                                     ILaTeXView)

        self.assertIn(
            r'\hyperref[path:/plone/the-book/the-chapter]{The Chapter'
            r'\footnote{See page'
            r' \pageref{path:/plone/the-book/the-chapter}}}',
            latex_view.render())

    def test_spaces_are_not_escaped(self):
        # Escaping spaces in URLs with %20 in LaTeX is bad because % is a comment.
        chapter = create(Builder('chapter')
                         .within(self.book)
                         .titled('the chapter'))

        document = create(Builder('file')
                          .within(chapter)
                          .titled('The File')
                          .with_id('the file'))

        html = '<a class="internal-link" href="resolveuid/{0}">Link</a>'.format(
            IUUID(document))
        block = create(Builder('book textblock')
                       .within(chapter)
                       .having(text=html))

        request = block.REQUEST
        assembler = getMultiAdapter((self.book, request), IPDFAssembler)
        latex_view = getMultiAdapter((block, request, assembler.get_layout()),
                                     ILaTeXView)

        self.assertEqual(
            r'\hyperref[path:/plone/the-book/the-chapter/the file]{Link'
            r'\footnote{See page'
            r' \pageref{path:/plone/the-book/the-chapter/the file}}}',
            latex_view.render().strip())



from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from StringIO import StringIO
from ftw.book.browser.reader.interfaces import IBookReaderRenderer
from ftw.book.interfaces import IBook
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.component import adapts
from zope.interface import implements, Interface
from zope.publisher.interfaces.browser import IBrowserView
import lxml.html


class BaseBookReaderRenderer(object):
    """Base block renderer used to subclass.
    """

    implements(IBookReaderRenderer)

    def __init__(self, context, request, readerview):
        self.context = context
        self.request = request
        self.readerview = readerview


class DefaultBlockRenderer(BaseBookReaderRenderer):
    """The simplelayout block renderer.
    It renders the simplealyout default "block_view".
    """

    adapts(ISimpleLayoutBlock, Interface, IBrowserView)

    def render(self):
        view = self.context.restrictedTraverse('block_view')
        html = view()
        html = self.mark_book_internal_links(html)
        return html

    def mark_book_internal_links(self, html):
        book = IBook(self.context)
        book_url = book.absolute_url()
        book_path = '/'.join(book.getPhysicalPath())
        context_url = self.context.absolute_url()

        doc = lxml.html.parse(StringIO(html))
        for node in doc.xpath('//a'):
            if 'href' not in node.attrib:
                continue

            url = self.resolve_uid(node.attrib['href'])
            if url == context_url:
                continue

            path = url.replace(book_url, book_path)
            uid = self.get_uid_by_path(path)
            if uid is None:
                continue

            if 'class' in node.attrib:
                node.attrib['class'] += ' book-internal'
            else:
                node.attrib['class'] = 'book-internal'

            node.attrib['data-uid'] = uid

        return lxml.html.tostring(doc)

    def get_uid_by_path(self, path):
        catalog = getToolByName(self.context, 'portal_catalog')
        rid = catalog.getrid(path)
        if rid is None:
            return None

        metadata = catalog.getMetadataForRID(rid)
        return metadata.get('UID', None)

    def resolve_uid(self, url):
        if '/' not in url:
            return url

        parts = url.split('/')
        if parts[-2] == 'resolveuid' or parts[-2] == 'resolveUid':
            reference_catalog = getToolByName(self.context,
                                              'reference_catalog')

            uid = parts[-1]
            obj = reference_catalog.lookupObject(uid)

            if obj is not None:
                url = obj.absolute_url()
        return url


class BookRenderer(BaseBookReaderRenderer):
    """The book renderer renders the title page and the table of
    contents of the book.
    """

    adapts(IBook, Interface, IBrowserView)

    template = ViewPageTemplateFile('templates/book_title.pt')

    def render(self):
        return self.template(title=self.context.Title())



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from unittest2 import TestCase



class TestChapter(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.book = create(Builder('book'))

    @browsing
    def test_creating_chapter(self, browser):
        browser.login().visit(self.book)
        factoriesmenu.add('Chapter')

        browser.fill({'Title': 'The Chapter'}).submit()
        self.assertEquals(self.book.absolute_url() + '/the-chapter/view',
                          browser.url)
        self.assertEquals('The Chapter', browser.css('h1').first.text)

    @browsing
    def test_subchapter_is_rendered_as_block_in_parent(self, browser):
        chapter = create(Builder('chapter')
                         .titled('Chapter')
                         .within(self.book))

        subchapter = create(Builder('chapter')
                            .titled('Sub Chapter')
                            .within(chapter))

        browser.login().visit(chapter)
        block, = browser.css('.BlockOverallWrapper.chapter')
        self.assertEquals(['1.1 Sub Chapter'], block.css('h3').text)

        block.find('1.1 Sub Chapter').click()
        self.assertEquals(subchapter, browser.context)



from contextlib import contextmanager
from ftw.book.interfaces import IBook
from ftw.book.interfaces import IWithinBookLayer
from ftw.pdfgenerator.utils import provide_request_layer
from zope.component import adapts
from zope.dottedname.resolve import resolve
from zope.interface import directlyProvidedBy, directlyProvides
from zope.publisher.interfaces import IRequest
from ZPublisher.BaseRequest import DefaultPublishTraverse


class BookTraverse(DefaultPublishTraverse):
    adapts(IBook, IRequest)

    def publishTraverse(self, request, name):
        provide_book_layers(self.context, request)
        return super(BookTraverse, self).publishTraverse(request, name)


def provide_book_layers(book, request):
    layout_layer_name = getattr(book, 'latex_layout', '')
    if layout_layer_name:
        layout_layer = resolve(layout_layer_name)
        provide_request_layer(request, layout_layer)
    provide_request_layer(request, IWithinBookLayer)


@contextmanager
def providing_book_layers(book, request):
    original_interfaces = list(directlyProvidedBy(request))
    provide_book_layers(book, request)
    try:
        yield
    finally:
        directlyProvides(request, *original_interfaces)



from ftw.book.interfaces import IWithinBookLayer
from ftw.book.latex.defaultlayout import IDefaultBookLayoutSelectionLayer
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.publisher.core.interfaces import IDataCollector
from unittest2 import TestCase
from zope.component import getAdapter
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides


class TestBookLayoutRequestLayerCollector(TestCase):
    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']

    def test_getData_sets_request_layers(self):
        book = create(Builder('book').with_layout(IDefaultBookLayoutSelectionLayer))

        self.remove_request_interfaces()
        self.assertFalse(IDefaultBookLayoutSelectionLayer.providedBy(self.request))
        self.assertFalse(IWithinBookLayer.providedBy(self.request))
        collector = getAdapter(book, IDataCollector, name='010-book-layout-request-layers')
        collector.getData()
        self.assertTrue(IDefaultBookLayoutSelectionLayer.providedBy(self.request))
        self.assertTrue(IWithinBookLayer.providedBy(self.request))

    def test_getData_returns_request_layer_dottednames(self):
        book = create(Builder('book').with_layout(IDefaultBookLayoutSelectionLayer))

        collector = getAdapter(book, IDataCollector, name='010-book-layout-request-layers')
        self.assertItemsEqual(
            ['ftw.book.interfaces.IWithinBookLayer',
             'ftw.book.latex.defaultlayout.IDefaultBookLayoutSelectionLayer'],
            collector.getData())

    def test_setData_sets_request_layers(self):
        book = create(Builder('book').with_layout(IDefaultBookLayoutSelectionLayer))

        collector = getAdapter(book, IDataCollector, name='010-book-layout-request-layers')
        data = collector.getData()

        self.remove_request_interfaces()
        self.assertFalse(IDefaultBookLayoutSelectionLayer.providedBy(self.request))
        self.assertFalse(IWithinBookLayer.providedBy(self.request))

        collector.setData(data, {})
        self.assertTrue(IDefaultBookLayoutSelectionLayer.providedBy(self.request))
        self.assertTrue(IWithinBookLayer.providedBy(self.request))

    def remove_request_interfaces(self):
        # Remove custom book interfaces from request
        to_remove = [IWithinBookLayer, IDefaultBookLayoutSelectionLayer]
        ifaces = [iface for iface in directlyProvidedBy(self.request)
                  if iface not in to_remove]
        directlyProvides(self.request, *ifaces)



from ftw.book.interfaces import IBook
from ftw.book.testing import FTW_BOOK_INTEGRATION_TESTING
from ftw.builder import Builder
from ftw.builder import create
from unittest2 import TestCase


class TestBookAdapterReturnsBook(TestCase):
    layer = FTW_BOOK_INTEGRATION_TESTING

    def setUp(self):
        self.book = create(Builder('book'))

    def test_on_book(self):
        self.assertEquals(self.book, IBook(self.book))

    def test_on_chapter(self):
        chapter = create(Builder('chapter').within(self.book))
        self.assertEquals(self.book, IBook(chapter))

    def test_on_text_block(self):
        chapter = create(Builder('chapter').within(self.book))
        block = create(Builder('book textblock').within(chapter))
        self.assertEquals(self.book, IBook(block))

    def test_on_file(self):
        chapter = create(Builder('chapter').within(self.book))
        block = create(Builder('file').within(chapter))
        self.assertEquals(self.book, IBook(block))



from Products.CMFCore.utils import getToolByName
from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from unittest2 import TestCase


class TestHTMLBlockViewOutsideOfBook(TestCase):
    layer = FTW_BOOK_FUNCTIONAL_TESTING

    @browsing
    def test_block_view_outside_of_book(self, browser):
        ttool = getToolByName(self.layer['portal'], 'portal_types')
        ttool['ContentPage'].allowed_content_types += ('HTMLBlock',)

        page = create(Builder('content page'))
        htmlblock = create(Builder('htmlblock')
                           .titled('HTML Block')
                           .having(showTitle=True,
                                   text='<p id="html-block-text">The Text</p>')
                           .within(page))

        browser.visit(htmlblock)
        self.assertEquals(page, browser.context,
                          'HTMLBlock should redirect to container.')

        self.assertEquals('HTML Block',
                          browser.css('.simplelayout-block-wrapper h2').first.text)
        self.assertEquals('The Text', browser.css('#html-block-text').first.text)



from Products.CMFCore.utils import getToolByName
from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from unittest2 import TestCase
import transaction


def keywords_html(*keywords):
    return '\n'.join(
        map(lambda word: '<span class="keyword" title="%s">%s</span>' % (
            word, word), keywords))


def select2_javascripts():
    sources = [node.attrib.get('src') for node in browser.css('script')
               if node.attrib.get('src')]
    return filter(lambda src: 'select2' in src, sources)


class TestKeywordsView(TestCase):
    layer = FTW_BOOK_FUNCTIONAL_TESTING

    def setUp(self):
        self.book = create(Builder('book').titled('The Book'))

    @browsing
    def test_keywords_only_available_when_use_keywords_enabled(self, browser):
        tab_label = 'Keywords'
        browser.login().visit(self.book, view='tabbed_view')
        self.assertNotIn(tab_label, browser.css('.tabbedview-tabs a').text)

        self.book.setUse_index(True)
        transaction.commit()

        browser.login().visit(self.book, view='tabbed_view')
        self.assertIn(tab_label, browser.css('.tabbedview-tabs a').text)

    @browsing
    def test_keywords_tab_is_available(self, browser):
        browser.login().visit(self.book, view='tabbedview_view-keywords')

    @browsing
    def test_keywords_tab_provides_select_with_keywords(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .having(text=keywords_html('Foo', 'bar', 'Baz')))

        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertItemsEqual(
            ['Foo', 'bar', 'Baz', ''],
            browser.css('select[name=book_keywords] option').text)

    @browsing
    def test_no_duplicate_keywords(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .having(text=keywords_html('Foo', 'bar', 'Foo')))

        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertItemsEqual(
            ['Foo', 'bar', ''],
            browser.css('select[name=book_keywords] option').text)

    @browsing
    def test_keywords_are_ordered_normalized_case_insensitive(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .having(text=keywords_html('foo',
                                          'bar',
                                          'Baz',
                                          '\xc3\x84hnliches',
                                          '\xc3\xb6rtliches',
                                          '\xc3\x9cbliches')))

        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertEquals(
            ['',
             u'\xc4hnliches',
             'bar',
             'Baz',
             'foo',
             u'\xf6rtliches',
             u'\xdcbliches'],
            browser.css('select[name=book_keywords] option').text)

    @browsing
    def test_load_results_by_keyword(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .having(text=keywords_html('Foo')))
        create(Builder('book textblock')
               .within(chapter)
               .having(text=keywords_html('Bar')))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')
        self.assertEquals(1, len(browser.css('.result')),
                          'Expected exactly one result')

    @browsing
    def test_message_when_there_are_no_results(self, browser):
        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals('No results found.',
                          browser.css('.no-results').first.text)

    @browsing
    def test_results_are_sorted_by_position_in_the_book(self, browser):
        one = create(Builder('chapter')
                     .titled('Chapter One')
                     .within(self.book))

        two = create(Builder('chapter')
                     .titled('Chapter Two')
                     .within(self.book))

        # Create blocks in reversed order for verifying that they are
        # actually sorted correctly.

        create(Builder('book textblock')
               .within(two)
               .titled('Block 2.2')
               .having(text=keywords_html('Foo'),
                       showTitle=True))

        block = create(Builder('book textblock')
                       .within(two)
                       .titled('Block 2.1')
                       .having(text=keywords_html('Foo'),
                               showTitle=True))
        two.moveObjectsByDelta([block.getId()], -1)

        create(Builder('book textblock')
               .within(one)
               .having(text=keywords_html('Foo')))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals(['1 Chapter One',
                           '2.1 Block 2.1',
                           '2.2 Block 2.2'],
                          browser.css('.result .title').text)

    @browsing
    def test_block_title_is_shown(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .titled('The Block')
               .having(text=keywords_html('Foo'),
                       showTitle=True))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals('1.1 The Block',
                          browser.css('.result .title').first.text)

    @browsing
    def test_chapter_title_used_when_block_title_invisible(self, browser):
        chapter = create(Builder('chapter')
                         .within(self.book)
                         .titled('The Chapter'))
        create(Builder('book textblock')
               .within(chapter)
               .titled('The Block')
               .having(text=keywords_html('Foo'),
                       showTitle=False))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals('1 The Chapter',
                          browser.css('.result .title').first.text)

    @browsing
    def test_title_is_linked_with_reader(self, browser):
        chapter = create(Builder('chapter')
                         .titled('The Chapter')
                         .within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .titled('The Block')
               .having(text=keywords_html('Foo'),
                       showTitle=True))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals(
            'http://nohost/plone/the-book/the-chapter/' +
            'the-block/@@book_reader_view',
            browser.find('1.1 The Block').attrib['href'])

    @browsing
    def test_keywords_are_shown_foreach_result(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .titled('The Block')
               .having(text=keywords_html('Foo', 'Bar', 'Baz'),
                       showTitle=True))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals(['Bar,', 'Baz,', 'Foo'],
                          browser.css('.result-keywords span').text)

        self.assertEquals(['Foo'],
                          browser.css('.result-keywords b').text)

    @browsing
    def test_no_duplicate_keywords_in_result(self, browser):
        chapter = create(Builder('chapter').within(self.book))
        create(Builder('book textblock')
               .within(chapter)
               .titled('The Block')
               .having(text=keywords_html('Foo', 'Bar', 'Foo'),
                       showTitle=True))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals(['Bar,', 'Foo'],
                          browser.css('.result-keywords span').text)

    @browsing
    def test_result_location_is_shown(self, browser):
        chapter = create(Builder('chapter')
                         .titled('Chapter')
                         .within(self.book))

        subchapter = create(Builder('chapter')
                            .titled('SubChapter')
                            .within(chapter))

        create(Builder('book textblock')
               .within(subchapter)
               .titled('The Block')
               .having(text=keywords_html('Foo'),
                       showTitle=True))

        browser.login().open(self.book,
                             {'book_keywords': 'Foo'},
                             view='tabbedview_view-keywords/load')

        self.assertEquals(['The Book', '1 Chapter', '1.1 SubChapter'],
                          browser.css('.result-location a').text)

        self.assertEquals(
            'http://nohost/plone/the-book/@@book_reader_view',
            browser.css('.result-location a').first.attrib['href'])

    def test_chapters_cache(self):
        first = create(Builder('chapter')
                       .titled('First Chapter')
                       .within(self.book))
        first_sub = create(Builder('chapter')
                           .titled('First SubChapter')
                           .within(first))
        create(Builder('chapter')
               .titled('First SubSubChapter')
               .within(first_sub))
        create(Builder('chapter')
               .titled('Second Chapter')
               .within(self.book))

        view = self.book.restrictedTraverse('tabbedview_view-keywords')

        self.assertItemsEqual(
            ['/plone/the-book',
             '/plone/the-book/first-chapter',
             '/plone/the-book/first-chapter/first-subchapter',
             '/plone/the-book/first-chapter/first-subchapter/' +
             'first-subsubchapter',
             '/plone/the-book/second-chapter'],
            view.chapters.keys())

        self.assertDictContainsSubset(
            {'reader_url': 'http://nohost/plone/the-book/@@book_reader_view',
             'title': 'The Book',
             'position': 0},
            view.chapters['/plone/the-book'])
        self.assertIn('brain', view.chapters['/plone/the-book'])

        self.assertDictContainsSubset(
            {'reader_url': 'http://nohost/plone/the-book/first-chapter/' +
             'first-subchapter/@@book_reader_view',
             'title': '1.1 First SubChapter',
             'position': 2},
            view.chapters['/plone/the-book/first-chapter/first-subchapter'])

    @browsing
    def test_only_search_for_results_in_this_book(self, browser):
        # Regression: when having the same keyword in multiple books
        # the keyword tab was broken because of unspecific query (no path).

        first_book = create(Builder('book').titled('First Book'))
        first_chapter = create(Builder('chapter').titled('First chapter')
                               .within(first_book))
        create(Builder('book textblock').titled('First Block')
               .within(first_chapter)
               .having(text=keywords_html('Foo', 'Bar')))

        second_book = create(Builder('book').titled('Second Book'))
        second_chapter = create(Builder('chapter').titled('Second chapter')
                                .within(second_book))
        create(Builder('book textblock').titled('Second Block')
               .within(second_chapter)
               .having(text=keywords_html('Bar', 'Baz')))

        browser.login().visit(first_book, view='tabbedview_view-keywords')
        self.assertItemsEqual(
            ['Foo', 'Bar', ''],
            browser.css('select[name=book_keywords] option').text,
            'Only keywords from the current book should be selectable.')

        browser.login().open(first_book,
                             {'book_keywords': 'Bar'},
                             view='tabbedview_view-keywords/load')
        self.assertEquals(1, len(browser.css('.result')),
                          'Only results from the current book should be'
                          ' found.')

    @browsing
    def test_select2_translations_are_loaded(self, browser):
        languages = getToolByName(self.layer['portal'], "portal_languages")

        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertEquals(
            ['++resource++ftw.book-select2/select2.js'],
            select2_javascripts(),
            'Expected no translation to be loaded for english.')

        languages.manage_setLanguageSettings('de', ['de'])
        transaction.commit()
        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertIn(
            '++resource++ftw.book-select2/select2_locale_de.js',
            select2_javascripts())

        languages.manage_setLanguageSettings('de-ch', ['de-ch'],
                                             setUseCombinedLanguageCodes=True)
        transaction.commit()
        browser.login().visit(self.book, view='tabbedview_view-keywords')
        self.assertIn(
            '++resource++ftw.book-select2/select2_locale_de.js',
            select2_javascripts())



from ftw.book.testing import EXAMPLE_CONTENT_INTEGRATION_TESTING
from ftw.book.tests.base import PDFDiffTestCase
import os


if not os.environ.get('SKIP_BOOK_EXPORTS', False):
    class TestPDFExport(PDFDiffTestCase):

        layer = EXAMPLE_CONTENT_INTEGRATION_TESTING
        book_object_path = 'example-book'
        expected_result = 'books/test_book_export.pdf'



from Products.Archetypes import atapi
from Products.Archetypes.public import BooleanField
from Products.Archetypes.public import StringField
from Products.Archetypes.public import TextField
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from ftw.book import _
from ftw.book.interfaces import IChapter
from ftw.book.interfaces import ILaTeXCodeInjectionEnabled
from ftw.book.interfaces import ITable
from ftw.book.interfaces import IWithinBookLayer
from ftw.book.interfaces import ModifyLaTeXInjection
from ftw.book.interfaces import NO_PREFERRED_LAYOUT
from ftw.book.interfaces import ONECOLUMN_LAYOUT
from ftw.book.interfaces import TWOCOLUMN_LAYOUT
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.component import adapts
from zope.interface import implements
import sys


class LaTeXCodeField(ExtensionField, TextField):
    pass


class ExtensionStringField(ExtensionField, StringField):
    pass


class ExtensionBooleanField(ExtensionField, BooleanField):
    pass


def add_field(field, condition=None, interfaces=None, insert_after=None):
    """Add a field under certain conditions.

    Arguments:

    field -- The schema extender field (subclassing ExtensionField)

    condition -- A function - called with context as argument - for deciding
    whether to inject the field or not.

    interfaces -- A list of interfaces from which at least one has to be
    provided by the context for the field to be applied.

    insert_after -- A string fieldname, after which this field is
    positioned, if both fields are in the same schemata.
    """

    cls = sys._getframe(1).f_locals
    cls['_fields'].append({'field': field,
                           'condition': condition,
                           'interfaces': interfaces,
                           'insert_after': insert_after})


def hide_from_toc_condition(context):
    field = context.schema.get('showTitle')
    if not field:
        return False

    visibility = field.widget.isVisible(context, 'edit')
    if visibility != 'visible':
        return False

    if ITable.providedBy(context):
        return False

    return True


class LaTeXCodeInjectionExtender(object):
    adapts(ILaTeXCodeInjectionEnabled)
    implements(IOrderableSchemaExtender)

    _fields = []

    add_field(LaTeXCodeField(
            name='preLatexCode',
            schemata='LaTeX',
            default_content_type='text/plain',
            allowable_content_types='text/plain',
            write_permission=ModifyLaTeXInjection,

            widget=atapi.TextAreaWidget(
                label=_(u'pre_latex_code_label',
                        default=u'LaTeX code above content'),
                description=_(u'pre_latex_code_help',
                              default=u''))))

    add_field(LaTeXCodeField(
            name='postLatexCode',
            schemata='LaTeX',
            default_content_type='text/plain',
            allowable_content_types='text/plain',
            write_permission=ModifyLaTeXInjection,

            widget=atapi.TextAreaWidget(
                label=_(u'post_latex_code_label',
                        default=u'LaTeX code beneath content'),
                description=_(u'post_latex_code_help',
                              default=u''))))

    add_field(ExtensionStringField(
            name='preferredColumnLayout',
            schemata='LaTeX',
            default=NO_PREFERRED_LAYOUT,
            write_permission=ModifyLaTeXInjection,
            vocabulary=((NO_PREFERRED_LAYOUT,
                         _('injection_label_no_preferred_column_layout',
                           default='No preferred column layout')),

                        (ONECOLUMN_LAYOUT,
                         _('injection_label_one_column_layout',
                           default='One column layout')),

                        (TWOCOLUMN_LAYOUT,
                         _('injection_label_two_column_layout',
                           default='Two column layout'))),

            widget=atapi.SelectionWidget(
                label=_(u'injection_label_preferred_column_layout',
                        default=u'Preferred layout'),
                description=_(
                    u'injection_help_preferred_column_layout',
                    default=u'When choosing a one or two column layout, the '
                    u'layout will switched for this content and all '
                    u'subsequent contents in the PDF, if necessary. '
                    u'If "no preferred layout" is selected the currently '
                    u'active layout is kept.'))))

    add_field(
        interfaces=[IChapter, ISimpleLayoutBlock],
        field=ExtensionBooleanField(
            name='latexLandscape',
            schemata='LaTeX',
            default=False,
            write_permission=ModifyLaTeXInjection,

            widget=atapi.BooleanWidget(
                label=_(u'injection_label_landscape',
                        default=u'Use landscape'))))

    add_field(
        interfaces=[IChapter, ISimpleLayoutBlock],
        field=ExtensionBooleanField(
            name='preLatexClearpage',
            schemata='LaTeX',
            default=False,
            write_permission=ModifyLaTeXInjection,

            widget=atapi.BooleanWidget(
                label=_(u'injection_label_insert_clearpage_before_content',
                        default=u'Insert page break before this content'))))

    add_field(
        interfaces=[IChapter, ISimpleLayoutBlock],
        field=ExtensionBooleanField(
            name='postLatexClearpage',
            schemata='LaTeX',
            default=False,
            write_permission=ModifyLaTeXInjection,

            widget=atapi.BooleanWidget(
                label=_(u'injection_label_insert_clearpage_after_content',
                        default=u'Insert page break after this content'))))

    add_field(
        interfaces=[IChapter, ISimpleLayoutBlock],
        field=ExtensionBooleanField(
            name='preLatexNewpage',
            schemata='LaTeX',
            default=False,
            write_permission=ModifyLaTeXInjection,

            widget=atapi.BooleanWidget(
                label=_(u'injection_label_insert_newpage_before_content',
                        default=u'Insert column break before this content'),
                description=_(u'This option inserts a column break when '
                              u'two column layout is active.'))))

    add_field(
        # hideFromTOC is only useful when we have a showTitle checkbox too
        condition=hide_from_toc_condition,
        insert_after='showTitle',
        field=ExtensionBooleanField(
            name='hideFromTOC',
            default=False,
            required=False,

            widget=atapi.BooleanWidget(
                label=_(u'injection_label_hide_from_toc',
                        default=u'Hide from table of contents'),
                description=_(u'injection_help_hide_from_toc',
                              default=u'Hides the title from the table of '
                              u'contents and does not number the heading.'))))

    def __init__(self, context):
        self.context = context

    def getFields(self):
        if not self._context_is_within_book():
            return []

        fields = []

        for item in self._fields:
            condition = item.get('condition')
            if condition and not condition(self.context):
                continue

            interfaces = item.get('interfaces')
            if interfaces:
                provided = [iface for iface in interfaces
                            if iface.providedBy(self.context)]
                if len(provided) == 0:
                    continue

            fields.append(item.get('field'))

        return fields

    def getOrder(self, schematas):
        for item in self._fields:
            insert_after = item.get('insert_after')
            if not insert_after:
                continue

            field = item.get('field')
            if field.schemata not in schematas:
                continue

            schemata = schematas[field.schemata]
            if insert_after not in schemata or field.__name__ not in schemata:
                continue

            schemata.remove(field.__name__)
            schemata.insert(schemata.index(insert_after) + 1, field.__name__)

        return schematas

    def _context_is_within_book(self):
        # In some cases REQUEST is no available.
        if not hasattr(self.context, 'REQUEST'):
            return False

        if IWithinBookLayer.providedBy(self.context.REQUEST):
            return True
        return False



from Acquisition import aq_inner, aq_parent
from ftw.book.interfaces import IBook
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot


class BookHelper(object):

    def __call__(self, obj, linked=False):
        return self.generate_title(obj, linked=linked)

    def generate_title(self, obj, linked=False):
        """ Generates a title embedded in a h-tag
        """

        if self._heading_is_numbered(obj):
            chapter_string = self.get_chapter_level_string(obj)
        else:
            chapter_string = ''

        title = obj.title_or_id()

        html = ' '.join((chapter_string, title)).strip()
        if linked:
            html = '<a href="%s">%s</a>' % (obj.absolute_url(), html)

        hierarchy = self.get_hierarchy_position(obj)
        html = "<h%s>%s</h%s>" % (
            hierarchy, html, hierarchy)

        return html

    def generate_valid_hierarchy_h_tags(self, obj):
        """ Generates the h-tags for parent-objects for valid html
        """
        titles = self._get_hierarchy_titles(obj)
        html = ""

        for i, title in enumerate(titles):
            html += '<h%s>%s</h%s>' % (i + 1, title, i + 1)
        return html

    def get_folder_position(self, obj):
        """ Get the position of the object in the folder
        """
        return self._get_filtered_folder_position(obj)

    def get_hierarchy_position(self, obj):
        """ Get the position in the hierarchy of the book
        """
        return self._get_hierarchy_position(obj)

    def get_chapter_level_string(self, obj):
        """ Return the string of the chapter levele: 1.4.3
        """
        chapter_level = self.get_chapter_level(obj)
        chapter_level = [str(level) for level in chapter_level]
        return '.'.join(chapter_level)

    def get_chapter_level(self, obj):
        """ Return the chapterlevel in a list
        of int
        """
        chapter_level = []
        parent = obj

        while not IPloneSiteRoot.providedBy(parent):
            if IBook.providedBy(parent):
                break
            else:
                chapter_level.insert(
                    0, self._get_filtered_folder_position(parent))
                parent = aq_parent(aq_inner(parent))

        return chapter_level

    def _get_hierarchy_titles(self, obj):
        """ Get all titles in the hierarchy in a list: ['first', 'second']
        """
        titles = []
        parent = obj
        while not IPloneSiteRoot.providedBy(parent):
            titles.insert(0, parent.Title())
            if IBook.providedBy(parent):
                break
            else:
                parent = aq_parent(aq_inner(parent))

        return titles

    def _get_hierarchy_position(self, obj):
        """ Get the hierarchy position of the object as int
        """

        position = len(self._get_hierarchy_titles(obj))
        return position < 6 and position or 6

    def _get_filtered_folder_position(self, obj):
        """ Return the filtered folder position as int
        """

        parent = aq_parent(aq_inner(obj))
        counter = 0

        folder_content = parent.contentValues()

        for item in folder_content:
            if self._heading_is_numbered(item):
                counter += 1

            if obj == item:
                break

        return counter

    def _heading_is_numbered(self, item):
        consider_types = ['Chapter']

        if item.portal_type in consider_types:
            return True

        if not hasattr(item, 'showTitle') or not item.showTitle:
            return False

        field = item.Schema().getField('hideFromTOC')
        if field and field.get(item):
            return False

        return True



from ftw.book.testing import FTW_BOOK_FUNCTIONAL_TESTING
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from unittest2 import TestCase


def toc_tree(item=None):
    if not item:
        item = browser.css('#content-core ul > li').first
    return (item.css('>div').first.text,
            map(toc_tree, item.css('>ul>li')))


class TestBookView(TestCase):

    layer = FTW_BOOK_FUNCTIONAL_TESTING

    @browsing
    def test_lists_table_of_contents(self, browser):
        book = create(Builder('book').titled('The Book'))
        chapter = create(Builder('chapter').titled('First Chapter')
                         .within(book))
        subchapter = create(Builder('chapter').titled('The SubChapter')
                            .within(chapter))
        create(Builder('book textblock').titled('Hidden Title Block')
               .having(showTitle=False).within(subchapter))
        create(Builder('book textblock').titled('Visible Title Block')
               .having(showTitle=True).within(subchapter))
        create(Builder('chapter').titled('Second Chapter').within(book))

        browser.login().visit(book)

        toc = ('The Book', [
                ('1 First Chapter', [
                        ('1.1 The SubChapter', [
                                ('1.1.1 Visible Title Block', [])])]),
                ('2 Second Chapter', [])])

        self.assertTupleEqual(toc, toc_tree())



from Products.ATContentTypes.interfaces.image import IATImage
from ftw.book.latex.utils import ImageLaTeXGenerator
from ftw.pdfgenerator.view import MakoLaTeXView
from simplelayout.base.interfaces import IBlockConfig
from zope.component import adapts
from zope.interface import Interface


class ImageLaTeXView(MakoLaTeXView):
    adapts(IATImage, Interface, Interface)

    def render(self):
        image = self.context.getImage()
        image_layout = IBlockConfig(self.context).image_layout
        caption = self.context.Description()
        generator = ImageLaTeXGenerator(self.context, self.layout)
        return generator(image, image_layout, floatable=False,
                         caption=caption)
